#include "data_thread.h"

image_formation::image_formation(int nNx, int nNy, QImage *nimage)
{
    sin_len = sizeof(client_sin);
    Nx = nNx;
    Ny = nNy;
    sp2.edit_scan_conv_image_size(nNx,nNy);
    sp2.edit_scan_conv_mechanism(pure_rotation);
    sp2.set_scan_conv_image_arbitrary();
    sp2.edit_LUT_gray_level(8);
    sp2.edit_LUT_quantification_type(linear);
    sp2.edit_LUT_bound(0.f, 512.f);
    sp2.edit_filter_type(no_filter);
    idim=false;
    sc_image.resize(Nx, Ny);
    filt_image.resize(Nx, Ny);
    lut_image.resize(Nx, Ny);
    intensity = false;
    lut_calculating = false;
    lut_editing = false;

    data_8b = nullptr;
    data_16b = nullptr;
    data = nullptr;
    qimage = nimage;
    port = 7539;
    void set_scan_off();
    id_line = 0;
    vx = nullptr;
    vraw = nullptr;
    venv = nullptr;

    // C fft
    dec = 1;
    pad_len = 1;
    tmp = nullptr;
    pad = nullptr;
    env = nullptr;
    set_fft_off();
    // C fft

    resize_vector(1,1);
}

image_formation::~image_formation()
{
    std::cout << "Destructor called" << std::endl;
    if (data_8b != nullptr)
    {
        delete [] data_8b;
        data_8b = nullptr;
    }
    if (data_16b != nullptr)
    {
        delete [] data_16b;
        data_16b = nullptr;
    }
    if (data != nullptr)
    {
        for (int i=0 ; i<Nline ; i++)
        {
            delete [] data[i];
            data[i] = nullptr;
        }
        delete [] data;
        data = nullptr;
    }
}

void image_formation::edit_port(int nport)
{
    port = nport;
}

void image_formation::edit_settings(const Settings &nsettings)
{
    if (Settings_get_value(nsettings, tcp_udp) == 0) //UDP
    {tcp = false;}
    else {tcp = true;}
    if (Settings_get_value(nsettings, number_of_bit) == 8)
    {bit_size = 8;}
    else {bit_size = 16;}
    int nNpoint = Settings_get_number_of_point(nsettings);
    int nNline = Settings_get_value(nsettings, number_of_line);
    resize_vector(nNline, nNpoint);
    // C fft
    dec = Settings_get_value(nsettings, decimation);
    sp2.edit_scan_conv_sampling(125.f, dec, 1540.f, nNpoint);
    sp2.edit_scan_conv_depth(0.f, 300.f);
    sp2.calcul_scan_conv_kernel();
}

void image_formation::resize_vector(int nNline, int nNpoint)
{
    std::cout << "resize vector" << std::endl;

    if (data_8b != nullptr) {delete [] data_8b;}
    if (data_16b != nullptr) {delete [] data_16b;}
    if (data != nullptr)
    {
        for (int i=0  ; i<Nline ; i++) {delete [] data[i];}
        delete [] data;
    }
    Npoint = nNpoint;
    Nline = nNline;
    if (bit_size == 8)
    {
        data_8b = new int8_t[Npoint+1];
        data_16b = new int16_t[1];
        data = new int*[Nline];
        for (int i=0 ; i<Nline ; i++) {data[i] = new int[Npoint];}
    }
    else
    {
        data_8b = new int8_t[1];
        data_16b = new int16_t[Npoint+1];
        data = new int*[Nline];
        for (int i=0 ; i<Nline ; i++) {data[i] = new int[Npoint];}
    }

    raw_image0.resize(Nline, Npoint);
    raw_image1.resize(Nline, Npoint);

    sp2.edit_scan_conv_us_image_parameters(Nline, Npoint);
    // C fft
    if (tmp != nullptr) {delete [] tmp;}
    if (pad != nullptr) {delete [] pad;}
    if (env != nullptr) {delete [] env;}
    int powd = 0;
    if (power_two(Npoint,&powd)){powd++;}
    pad_len=int_pow(2,powd);
    tmp = new double[pad_len];
    pad = new double[pad_len];
    env = new double[pad_len];
    if (vraw != nullptr)
    {
        vraw->resize(Npoint);
    }
    if (venv != nullptr)
    {
        venv->resize(Npoint);
    }
    if (vx != nullptr)
    {
        vx->resize(Npoint);
        for (int i=0 ; i<Npoint ; i++)
        {
            (*vx)[i] = i;
        }
    }
}
void image_formation::launch_client()
{
    if (!tcp) //UDP
    {
        init_UDP_server_no_validation(&server_socket, &client_sin, port);
    }
    else //TCP
    {
        init_TCP_server_simple(&server_socket, &client_socket, &client_sin, port);
    }
}

void image_formation::close_client()
{
    if (!tcp) //UDP
    {
        close_UDP_server(&server_socket);
    }
    else
    {
        close_TCP_server_simple(&server_socket, &client_socket);
    }
}

void image_formation::data_stream()
{
    // C fft
    init_table(pad_len);
    float fech, f0, fm;
    fech=125000000.f/float(dec);
    f0=1500000.f;
    fm=5500000.f;
    // C fft

    launch_client();
    int old_line=0, new_line=0;
    while(scan)
    {
        if(bit_size == 8)
        {
            if(!tcp) //UDP
            {
                recvfrom(server_socket, (char *)data_8b, Npoint+1, 0, (SOCKADDR *)(&client_sin), &sin_len);
            }
            else
            {
                recv(client_socket, (char *)data_8b, (Npoint+1), MSG_WAITALL);
            }
            new_line = int(data_8b[0]);

            if (new_line == id_line)
            {
                for (int i=0 ; i<Npoint ; i++)
                {
                    tmp[i] = double(data_8b[i+1]);
                    (*vraw)[i] = double(data_8b[i+1]);
                }
                zero_padding(tmp, pad, Npoint, pad_len, 1);
                envelope(pad, env, pad_len, fech, f0, fm, 0);
                for (int i=0 ; i<Npoint ; i++)
                {
                    (*venv)[i] = env[i];
                }
                emit plot_ready();
            }

            if (fft)
            {
                for (int i=0 ; i<Npoint ; i++)
                {
                    tmp[i] = double(data_8b[i+1]);
                }
                zero_padding(tmp, pad, Npoint, pad_len, 1);
                envelope(pad, env, pad_len, fech, f0, fm, 0);
                if (idim)
                {
                    for (int i=0 ; i<Npoint ; i++)
                    {
                        raw_image0(new_line,i) = std::abs(float(env[i]));
                    }
                }
                else
                {
                    for (int i=0 ; i<Npoint ; i++)
                    {
                        raw_image1(new_line,i) = std::abs(float(env[i]));
                    }
                }
            }
            else
            {
                if (idim)
                {
                    for (int i=0 ; i<Npoint ; i++)
                    {
                        raw_image0(new_line,i) = std::abs(float(data_8b[i]));
                    }
                }
                else
                {
                    for (int i=0 ; i<Npoint ; i++)
                    {
                        raw_image1(new_line,i) = std::abs(float(data_8b[i]));
                    }
                }
            }
        }
        else
        {
            if(!tcp) //UDP
            {
                recvfrom(server_socket, (char *)data_16b, 2*(Npoint+1), 0, (SOCKADDR *)(&client_sin), &sin_len);
            }
            else
            {
                recv(client_socket, (char *)data_16b, 2*(Npoint+1), MSG_WAITALL);
            }
            new_line = int(data_16b[0]);

            if (new_line == id_line)
            {
                for (int i=0 ; i<Npoint ; i++)
                {
                    tmp[i] = double(data_16b[i+1]);
                    (*vraw)[i] = double(data_16b[i+1]);
                }
                zero_padding(tmp, pad, Npoint, pad_len, 1);
                envelope(pad, env, pad_len, fech, f0, fm, 0);
                for (int i=0 ; i<Npoint ; i++)
                {
                    (*venv)[i] = env[i];
                }
                emit plot_ready();
            }

            if (fft)
            {
                for (int i=0 ; i<Npoint ; i++)
                {
                    tmp[i] = double(data_16b[i+1]);
                }
                zero_padding(tmp, pad, Npoint, pad_len, 1);
                envelope(pad, env, pad_len, fech, f0, fm, 0);
                if (idim)
                {
                    for (int i=0 ; i<Npoint ; i++)
                    {
                        if (intensity)
                        {
                            raw_image0(new_line,i) = std::abs(20.f*std::log10(float(env[i])));
                        }
                        else
                        {
                            raw_image0(new_line,i) = std::abs(float(env[i]));
                        }
                    }
                }
                else
                {
                    for (int i=0 ; i<Npoint ; i++)
                    {
                        if (intensity)
                        {
                            raw_image1(new_line,i) = std::abs(20.f*std::log10(float(env[i])));
                        }
                        else
                        {
                            raw_image1(new_line,i) = std::abs(float(env[i]));
                        }
                    }
                }
            }
            else
            {
                if (idim)
                {
                    for (int i=0 ; i<Npoint ; i++)
                    {
                        if (intensity)
                        {
                            raw_image0(new_line,i) = std::abs(20.f*std::log10(float(data_16b[i])));
                        }
                        else
                        {
                            raw_image0(new_line,i) = std::abs(float(data_16b[i]));
                        }
                    }
                }
                else
                {
                    for (int i=0 ; i<Npoint ; i++)
                    {
                        if (intensity)
                        {
                            raw_image1(new_line,i) = std::abs(20.f*std::log10(float(data_16b[i])));
                        }
                        else
                        {
                            raw_image1(new_line,i) = std::abs(float(data_16b[i]));
                        }
                    }
                }
            }
        }
        //std::cout << "from data_thread, line: " << new_line << std::endl;

        if ( (old_line == Nline-2) && (new_line == Nline-1) && (qimage != nullptr))
        {
            //std::cout << "from_data_thread: image calculation" << std::endl;
            if (idim)
            {
                sp2.apply_scan_conv(raw_image0, sc_image);
            }
            else
            {
                sp2.apply_scan_conv(raw_image1, sc_image);
            }
            idim = !idim;
            sp2.apply_filter(sc_image, filt_image);
            if (lut_is_editing())
            {
                while (lut_is_editing())
                {
                    usleep(10);
                }
            }
            lut_calculating_on();
            sp2.apply_LUT(filt_image, lut_image);
            lut_calculation_off();
            for (int i=0 ; i<Nx ; i++)
            {
                for (int j=0 ; j<Ny ; j++)
                {
                    qimage->setPixel(i,j,qRgb(lut_image(i,j),lut_image(i,j),lut_image(i,j)));
                }
            }
            emit image_ready();
        }
        if ( (old_line == 1) && (new_line == 0) && (qimage != nullptr))
        {            
            //std::cout << "from_data_thread: image calculation" << std::endl;
            if (idim)
            {
                sp2.apply_scan_conv(raw_image0, sc_image);
            }
            else
            {
                sp2.apply_scan_conv(raw_image1, sc_image);
            }
            idim = !idim;
            sp2.apply_filter(sc_image, filt_image);
            sp2.apply_LUT(filt_image, lut_image);
            for (int i=0 ; i<Nx ; i++)
            {
                for (int j=0 ; j<Ny ; j++)
                {
                    qimage->setPixel(i,j,qRgb(lut_image(i,j),lut_image(i,j),lut_image(i,j)));
                }
            }
            emit image_ready();
        }

        old_line = new_line;
    }
    close_client();
    emit finished();
}
