#ifndef DATA_THREAD_H
#define DATA_THREAD_H

#include <QObject>
#include <QImage>
#include <QVector>
#include <QThread>
#include <QtDebug>
#include <iostream>

extern "C"
{
    #include "settings.h"
    #include "TCP_API.h"
    #include "UDP_API.h"
    #include "signal_processing.h"
}
#include"SigProc2D.hpp"
#include"image.hpp"

// based on https://qt-labs.developpez.com/thread/qthread-sans-maux-tete/

class image_formation : public QObject
{
    Q_OBJECT // mandatory for class that use its own signal slot

    public:
        SigProc2D sp2 = SigProc2D();
        image_formation(int nNx=256, int nNy=256, QImage *nimage = nullptr);
        ~image_formation();

        void edit_port(int nport);
        void edit_settings(const Settings &nsettings);
        void set_scan_on() {scan = true;}
        void set_scan_off() {scan = false;}

        void add_image(QImage *nimage) {qimage = nimage;}
        void edit_id_line(int nid_line) {id_line = nid_line;}
        void add_x_vector(QVector<double> *nx) {vx = nx;}
        void add_raw_vector(QVector<double> *nraw_vector) {vraw = nraw_vector;}
        void add_env_vector(QVector<double> *nenv_vector) {venv = nenv_vector;}

        void set_fft_on() {fft = true;}
        void set_fft_off() {fft = false;}
        void set_intensity() {intensity = true;}
        void set_pressure() {intensity = false;}
        void lut_calculating_on() {lut_calculating = true;}
        void lut_calculation_off() {lut_calculating = false;}
        bool lut_is_calculating() {return lut_calculating;}
        void lut_editing_on() {lut_editing = true;}
        void lut_editing_off() {lut_editing = false;}
        bool lut_is_editing() {return lut_editing;}

    private:
        int Nline;
        int Npoint;
        int Nx;
        int Ny;
        QImage *qimage;
        QVector<double> *vx;
        QVector<double> *vraw;
        QVector<double> *venv;
        int id_line;

        image<float> raw_image0 = image<float>();
        image<float> raw_image1 = image<float>();
        image<float> sc_image = image<float>();
        image<float> filt_image = image<float>();
        image<int> lut_image = image<int>();
        bool idim;
        bool intensity;
        bool lut_calculating;
        bool lut_editing;

        // data vectors
        int bit_size;
        int8_t *data_8b;
        int16_t *data_16b;
        int **data;

        // socket parameters
        bool scan;
        int port;
        SOCKET server_socket;
        SOCKET client_socket;
        SOCKADDR_IN client_sin;
        socklen_t sin_len;
        bool tcp;

        void resize_vector(int nNline, int nNpoint);
        void launch_client();
        void close_client();

        //for C fft
        int dec;
        int pad_len; //zeros padding length
        double *tmp; // temporary buffer to store line data, size Npoint
        double *pad; // buff to store fft(tmp) with zero padding, size pad_len
        double *env; //buff to store the envelope of the line, size pad_len
        bool fft;

    signals:
        void image_ready();
        void plot_ready();
        void finished();

    public slots:
        void data_stream();
};

class data_thread : public QThread
{
protected:
    void run() { exec(); }
};

#endif // DATA_THREAD_H
