#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);
    connect = false;
    ui->widget_connect->setStyleSheet("background-color:red;");
    ui->widget_connect->show();
    ui->tabWidget->setCurrentIndex(0);
    scan = false;

    // image
    image = QImage (256, 256, QImage::Format_Grayscale8);
    for (int i=0 ; i<256 ; i++)
    {
        for (int j=0 ; j<256 ; j++)
        {
            image.setPixel(i,j,qRgb(0,0,0));
        }
    }
    graph = new QGraphicsScene (this);
    graph->addPixmap(QPixmap::fromImage(image));
    ui->graphicsView->setScene(graph);

    //plot
    ui->widget_plot->addGraph();
    ui->widget_plot->graph(0)->setPen(QPen(Qt::blue));
    ui->widget_plot->addGraph();
    ui->widget_plot->graph(1)->setPen(QPen(Qt::red));
    QVector<double> x(251), y0(251), y1(251);
    for (int i=0; i<251; ++i)
    {
      x[i] = i;
      y0[i] = qExp(-i/150.0)*qCos(i/10.0); // exponentially decaying cosine
      y1[i] = qExp(-i/150.0);              // exponential envelope
    }
    ui->widget_plot->graph(0)->setData(x, y0);
    ui->widget_plot->graph(1)->setData(x, y1);
    // let the ranges scale themselves so graph 0 fits perfectly in the visible area:
    ui->widget_plot->graph(0)->rescaleAxes();
    // same thing for graph 1, but only enlarge ranges (in case graph 1 is smaller than graph 0):
    ui->widget_plot->graph(1)->rescaleAxes(true);
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::edit_ui_from_settings()
{
    if ( Settings_get_value(client_settings, tcp_udp) == 0)//UDP
    {
        ui->radioButton_UDP->setChecked(true);
        // init udp server?
    }
    else
    {
        ui->radioButton_TCP->setChecked(true);
    }
    if (Settings_get_value(client_settings, number_of_bit) <= 8)
    {
        ui->radioButton_8bit->setChecked(true);
    }
    else
    {
        ui->radioButton_12bit->setChecked(true);
    }
    if (Settings_get_value(client_settings, decimation) < 8)
    {
        ui->radioButton_dec1->setChecked(true);
    }
    else
    {
        ui->radioButton_dec8->setChecked(true);
    }
    if (Settings_get_value(client_settings, emulator) == 0)
    {
        ui->radioButton_measurement->setChecked(true);
    }
    else
    {
        ui->radioButton_emulator->setChecked(true);
    }
    Nline = Settings_get_value(client_settings, number_of_line);
    ui->lineEdit_Nline->setText(QString::number(Nline));
    Npoint = Settings_get_number_of_point(client_settings);
    ui->lineEdit_Npoint->setText(QString::number(Npoint));
    ui->lineEdit_delay->setText(QString::number(Settings_get_delay(client_settings)));
    ui->lineEdit_angle->setText(QString::number(Settings_get_value(client_settings, angle)));
    double tgc = double(Settings_get_value(client_settings, tgc_start))/255.0;
    tgc = -4.5+48.0*tgc;
    ui->lineEdit_tgc_start->setText(QString::number(tgc,'f',2));
    tgc = double(Settings_get_value(client_settings, tgc_end))/255.0;
    tgc = -4.5+48.0*tgc;
    ui->lineEdit_tgc_stop->setText(QString::number(tgc,'f',2));
}

void MainWindow::edit_settings_from_ui()
{
    if (ui->radioButton_TCP->isChecked())
    {
        Settings_edit(&client_settings, tcp_udp, 1);
    }
    else
    {
        Settings_edit(&client_settings, tcp_udp, 0);
    }
    if (ui->radioButton_8bit->isChecked())
    {
        Settings_edit(&client_settings, number_of_bit, 8);
    }
    else
    {
        Settings_edit(&client_settings, number_of_bit, 16);
    }
    if (ui->radioButton_dec1->isChecked())
    {
        Settings_edit(&client_settings, decimation, 1);
    }
    else
    {
        Settings_edit(&client_settings, decimation, 8);
    }
    if (ui->radioButton_emulator->isChecked())
    {
        Settings_edit(&client_settings, emulator, 1);
    }
    else
    {
        Settings_edit(&client_settings, emulator, 0);
    }
    QString data = "";
    float tgc = 0.f;
    data = ui->lineEdit_Nline->text();
    Settings_edit(&client_settings, number_of_line, data.toInt());
    data = ui->lineEdit_Npoint->text();
    Settings_edit_number_of_point(&client_settings, data.toInt());
    data = ui->lineEdit_delay->text();
    Settings_edit_delay(&client_settings, data.toInt());
    data = ui->lineEdit_angle->text();
    Settings_edit(&client_settings, angle, data.toInt());
    data = ui->lineEdit_tgc_start->text();
    tgc = data.toFloat();
    Settings_edit(&client_settings, tgc_start, int(255.f/48.f*(tgc+4.5f)));
    data = ui->lineEdit_tgc_stop->text();
    tgc = data.toFloat();
    Settings_edit(&client_settings, tgc_end, int(255.f/48.f*(tgc+4.5f)));
    Settings_edit_buffer_from_settings(client_settings, settings_buffer);
}

void MainWindow::on_pushButton_Connect_clicked()
{
    QString qs_IP = ui->lineEdit_IP->text();
    const char* IP= qs_IP.toStdString().c_str();
    QString qs_sock = ui->lineEdit_settings_socket->text();
    int settings_port = qs_sock.toInt();

    init_TCP_client(&settings_socket, IP, settings_port);
    connect = true;
    ui->widget_connect->setStyleSheet("background-color:green;");
    receive_TCP_client(&settings_socket, (char *)settings_buffer, 2*settings_size);
    Settings_edit_from_TCP_buffer(&client_settings, settings_buffer);
    edit_ui_from_settings();
}

void MainWindow::on_pushButton_Disconnect_clicked()
{
    if (connect)
    {
        close_TCP_client(&settings_socket);
        connect = false;
        ui->widget_connect->setStyleSheet("background-color:red;");
    }
}


void MainWindow::on_pushButton_send_settings_clicked()
{
    edit_settings_from_ui();
    send_TCP_client(&settings_socket, (char *)settings_buffer, 2*settings_size);
    receive_TCP_client(&settings_socket, (char *)settings_buffer, 2*settings_size);
    Settings_edit_from_TCP_buffer(&client_settings, settings_buffer);
    edit_ui_from_settings();
}

void MainWindow::display_image()
{
    graph->addPixmap( QPixmap::fromImage(image) );
    ui->graphicsView->setScene(graph);
}

void MainWindow::display_plot()
{
    ui->widget_plot->graph(0)->setData(lx, lraw);
    ui->widget_plot->graph(1)->setData(lx, lenv);
    // let the ranges scale themselves so graph 0 fits perfectly in the visible area:
    ui->widget_plot->graph(0)->rescaleAxes();
    // same thing for graph 1, but only enlarge ranges (in case graph 1 is smaller than graph 0):
    ui->widget_plot->graph(1)->rescaleAxes(true);
    ui->widget_plot->replot();
}

void MainWindow::on_pushButton_start_clicked()
{
 /*   if (!scan)
    {
        scan = true;
        std::thread receive_data(&MainWindow::data_thread,this);
        receive_data.detach();
    }*/
    Settings_edit(&client_settings, start, 1);
    Settings_edit_buffer_from_settings(client_settings, settings_buffer);
    send_TCP_client(&settings_socket, (char *)settings_buffer, 2*settings_size);
    receive_TCP_client(&settings_socket, (char *)settings_buffer, 2*settings_size);

    lim_form.add_image(&image);
    lim_form.add_x_vector(&lx);
    lim_form.add_raw_vector(&lraw);
    lim_form.add_env_vector(&lenv);
    lim_form.edit_settings(client_settings);
    lim_form.set_scan_on();
    lim_form.moveToThread(&ldat_thr);
    lim_form.connect(&ldat_thr,SIGNAL(started()),SLOT(data_stream()));
    ldat_thr.connect(&lim_form,SIGNAL(finished()),SLOT(quit()));
    QObject::connect(&lim_form,SIGNAL(image_ready()),this,SLOT(display_image()));
    QObject::connect(&lim_form,SIGNAL(plot_ready()),this,SLOT(display_plot()));

    ldat_thr.start();
}

void MainWindow::on_pushButton_stop_clicked()
{
    //if (scan)
    //{
        //scan = false;
        Settings_edit(&client_settings, start, 0);
        Settings_edit_buffer_from_settings(client_settings, settings_buffer);
        send_TCP_client(&settings_socket, (char *)settings_buffer, 2*settings_size);
        receive_TCP_client(&settings_socket, (char *)settings_buffer, 2*settings_size);
        lim_form.set_scan_off();
        usleep(1000000);
        if (!ldat_thr.isFinished()) {ldat_thr.quit();}
    //}
}

void MainWindow::on_comboBox_power_currentIndexChanged(int index)
{
    if (index == 0)
    {
        lim_form.set_pressure();
    }
    else
    {
        lim_form.set_intensity();
    }
}

void MainWindow::on_checkBox_fft_stateChanged(int arg1)
{
    if (Qt::CheckState(arg1) == Qt::Checked )
    {
        lim_form.set_fft_on();
    }
    else
    {
        lim_form.set_fft_off();
    }
}

void MainWindow::on_comboBox_lut_type_currentIndexChanged(const QString &arg1)
{
    while (lim_form.lut_is_calculating())
    {
        usleep(10);
    }
    lim_form.lut_editing_on();
    if (arg1 == "linear")
    {
        lim_form.sp2.edit_LUT_quantification_type(linear);
    }
    else if (arg1 == "logarithmic")
    {
        lim_form.sp2.edit_LUT_quantification_type(logaritmic);
    }
    else if (arg1 == "exponential")
    {
        lim_form.sp2.edit_LUT_quantification_type(exponential);
    }
    else
    {
        lim_form.sp2.edit_LUT_quantification_type(sigmoid);
    }
    lim_form.sp2.init_LUT();
    lim_form.lut_editing_off();
}

void MainWindow::on_horizontalSlider_lbound_valueChanged(int value)
{
    int hvalue = ui->horizontalSlider_ubound->value();
    if (value >= hvalue)
    {
        value = hvalue - 1;
        ui->horizontalSlider_lbound->setValue(value);
    }
    while (lim_form.lut_is_calculating())
    {
        usleep(10);
    }
    lim_form.lut_editing_on();
    lim_form.sp2.edit_LUT_bound(value, ui->horizontalSlider_ubound->value());
    lim_form.sp2.init_LUT();
    lim_form.lut_editing_off();
    ui->horizontalSlider_lbound->setToolTip(QString::number(value));
}

void MainWindow::on_horizontalSlider_ubound_valueChanged(int value)
{
    int lvalue = ui->horizontalSlider_lbound->value();
    if (value <= lvalue)
    {
        value = lvalue + 1;
        ui->horizontalSlider_ubound->setValue(value);
    }
    while (lim_form.lut_is_calculating())
    {
        usleep(10);
    }
    lim_form.lut_editing_on();
    lim_form.sp2.edit_LUT_bound(ui->horizontalSlider_lbound->value(),value);
    lim_form.sp2.init_LUT();
    lim_form.lut_editing_off();
    ui->horizontalSlider_lbound->setToolTip(QString::number(value));
}

void MainWindow::on_lineEdit_lbound_lv_textEdited(const QString &arg1)
{
    std::cout << arg1.toStdString() << std::endl;
    int lvalue = arg1.toInt();
    int hvalue = ui->lineEdit_lbound_hv->text().toInt();
    if (lvalue >= hvalue-10) {lvalue = hvalue-10;}
    int value = ui->horizontalSlider_lbound->value();
    if (value < lvalue) {value = lvalue;}
    ui->horizontalSlider_lbound->setMinimum(lvalue);
    ui->horizontalSlider_lbound->setValue(value);
}

void MainWindow::on_lineEdit_lbound_hv_textEdited(const QString &arg1)
{

}

void MainWindow::on_lineEdit_textEdited(const QString &arg1)
{
    int nid_line = arg1.toInt();
    lim_form.edit_id_line(nid_line);
}
