#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QGraphicsScene>
#include <iostream>
#include <thread>
#include <qmath.h>

extern "C"
{
    #include "settings.h"
    #include "TCP_API.h"
    #include "UDP_API.h"
    //#include "signal_processing.h"
}
#include "data_thread.h"
#include "qcustomplot.h"

namespace Ui {
class MainWindow;
}

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = nullptr);
    ~MainWindow();

signals:
    void new_image();

private slots:
    void on_pushButton_Connect_clicked();
    void on_pushButton_Disconnect_clicked();
    void on_pushButton_send_settings_clicked();
    void on_pushButton_start_clicked();
    void on_pushButton_stop_clicked();
    void display_image();
    void display_plot();

    void on_comboBox_power_currentIndexChanged(int index);
    void on_checkBox_fft_stateChanged(int arg1);
    void on_comboBox_lut_type_currentIndexChanged(const QString &arg1);
    void on_horizontalSlider_lbound_valueChanged(int value);
    void on_horizontalSlider_ubound_valueChanged(int value);
    void on_lineEdit_lbound_lv_textEdited(const QString &arg1);
    void on_lineEdit_lbound_hv_textEdited(const QString &arg1);
    void on_lineEdit_textEdited(const QString &arg1);

private:
    Ui::MainWindow *ui;
    QImage image;
    QGraphicsScene *graph;
    QVector<double> lx, lraw, lenv;
    image_formation lim_form;
    data_thread ldat_thr;

    bool connect;
    SOCKET settings_socket;
    Settings client_settings;
    uint8_t settings_buffer[2*settings_size];
    int Npoint;
    int Nline;

    // server parameters for data
    bool scan;
    SOCKET server_socket;
    SOCKET client_socket;
    SOCKADDR_IN client_sin;

    void edit_ui_from_settings();
    void edit_settings_from_ui();
};

#endif // MAINWINDOW_H
