#ifndef FILTER_HPP
#define FILTER_HPP

#include <memory>
#include "ThreadPool.h"
#include "image.hpp"
//#include "../json/json.hpp"

enum FilterType
{
    no_filter = 0,
    convolution,
    median
};

class Filter
{
  public:
    Filter();
    ~Filter(){}

    void setUp(std::shared_ptr<ThreadPool> iThreadPool);
    void print();
    FilterType get_filter_type() {return filter_type;}
    void edit_filter_type(FilterType nfilter_type);
    void edit_number_of_packet(int np) {NumberOfPackets = np;}
    int get_number_of_packet() {return NumberOfPackets;}
    image<float> get_kernel() {return mKernel;}
    
    //Convolution filter
    void reset_kernel();
    void edit_kernel(image<float> iKernel);
    void edit_kernel(float f00, float f01, float f02, float f10, float f11, float f12, float f20, float f21, float f22);
    void normalise_kernel();
    
    //Median filter
    
    //General filter function
    void NoFilter(const image<float> &iImageInput, image<float> &oImageOutput);
    void applyConvolutionFilter(const image<float> &iImageInput, image<float> &oImageOutput);
    void applyMedianFilter(const image<float> &iImageInput, image<float> &oImageOutput);
    void applyFilter(const image<float> &iImageInput, image<float> &oImageOutput);


  private:
    int NumberOfPackets;
    std::shared_ptr<ThreadPool> mThreadPool;
    FilterType filter_type; //0 no filter, 1 convolutuion filter, 2 median filter
    
    //Convolution filter parameters
    float normalisation;
    float normalisation2x3top;
    float normalisation2x3bottom;
    float normalisation3x2left;
    float normalisation3x2right;
    image<float> mKernel = image<float>(3,3);

    //Convolution filter functions
    void partial_convolution_filter (const image<float> &ImageInput, image<float> &ImageOutput, int StartLine, int EndLine);
    float getWeightedValueOn2x2Sector(const image<float> &iImage, int iX, int iY) const; //const is not mandatory
    float getWeightedValueOn2x3Sector(const image<float> &iImage, int iX, int iY) const;
    float getWeightedValueOn3x2Sector(const image<float> &iImage, int iX, int iY) const;
    float getWeightedValueOn3x3Sector(const image<float> &iImage, int iX, int iY) const;
    
    //Median filter functions
    void partial_median_filter(const image<float> &ImageInput, image<float> &ImageOutpu, int StartLine, int EndLine);
    float getMedianOn2x2Neighboors(float ** iImage, int iTop, int iLeft, std::vector<float> &lNeighboors) const;
    float getMedianOn2x2Neighboors(image<float> iImage, int iTop, int iLeft, std::vector<float> &lNeighboors) const;
    float getMedianOn2x3Neighboors(float ** iImage, int iTop, int iLeft, std::vector<float> &lNeighboors) const;
    float getMedianOn2x3Neighboors(image<float> iImage, int iTop, int iLeft, std::vector<float> &lNeighboors) const;
    float getMedianOn3x2Neighboors(float ** iImage, int iTop, int iLeft, std::vector<float> &lNeighboors) const;
    float getMedianOn3x2Neighboors(image<float> iImage, int iTop, int iLeft, std::vector<float> &lNeighboors) const;
    float getMedianOn3x3Neighboors(float ** iImage, int iTop, int iLeft, std::vector<float> &lNeighboors) const;
    float getMedianOn3x3Neighboors(image<float> iImage, int iTop, int iLeft, std::vector<float> &lNeighboors) const;

};

#endif
