#ifndef GRAYSCALE_HPP
#define GRAYSCALE_HPP

#include <memory>

#include "ThreadPool.h"
#include "image.hpp"
//#include "../json/json.hpp"

enum QuantificationType
{
    linear = 0,
    logaritmic,
    exponential,
    sigmoid
};

class GrayScale
{
  public:
    GrayScale(float nLowerBound = 0.f, float nUpperBound = 2047.f, int nGrayLevel = 8, QuantificationType nType = linear);
    ~GrayScale(){}
    void print();

    void edit_Bound(float nLowerBound = 0.f, float UpperBound = 2047.f);
    void edit_GrayLevel(int nGrayLevel = 8);
    void edit_QuantificationType(QuantificationType nType = linear);
    void setUp(std::shared_ptr<ThreadPool> iThreadPool);
    void edit_number_of_packet(int np) {NumberOfPackets = np;}
    void initFilter();
    void applyFilter(const image<float> &iImageInput, image<int> &oImageOutput);
    
    float get_lower_bound() {return LowerBound;}
    float get_upper_bound() {return UpperBound;}
    QuantificationType get_quantification_type() {return Type;}
    int get_gray_level() {return GrayLevel;}
    int get_number_of_packets() {return NumberOfPackets;}

    int get_output(float value);

  private:
    const static int MinPixelValue = 0;
    const static int MaxPixelValue = 255;
    const float Exponential_factor = 3.f; // change the "curve" of exponential LUT, if increase the distance with y=x function
    float LowerBound;  //if input <= LowerBound, output = 0
    float UpperBound; //if input >= UpperBoune, output = 255
    float OutMax;
    float OutMin;
    int GrayLevel; // Number of bits to encode gray level
    float Scale; //factor used to have output in [0 255]
    float Slope;
    float InternalSlope; //for linear interpolation of absiss for sigmoid
    QuantificationType Type;

    int NumberOfPackets;

    std::shared_ptr<ThreadPool> mThreadPool;
    
    int LinearQuantification(float value);
    int LogarithmicQuantification(float value);
    int ExponentialQuantification(float value);
    int SigmoidQuantification(float value);
};


#endif
