#include<iostream>
#include<cmath>

#include"SigProc1D.hpp"

SigProc1D::SigProc1D(int nNel)// : Raw_data(nNel), Filtered_data(nNel), Gain(nNel), Input_gain(1)
{
    Nel = nNel;
    Raw_data.resize(Nel);
    Filtered_data.resize(Nel);
    Gain.resize(Nel);
    Input_gain.resize(1);
}

//~SigProc1D::SigProc1D()

void SigProc1D::print()
{
    std::cout << "Size: " << Nel << std::endl;
    std::cout << "Raw data: " << Raw_data;
    std::cout << "Filtered data: " << Filtered_data;
    std::cout << "Gain: " << Gain;
    std::cout << "Input gain: " << Input_gain;
}

void SigProc1D::resize(int nNel)
{
    if (nNel != Nel)
    {
        Nel = nNel;
        Raw_data.resize(Nel);
        Filtered_data.resize(Nel);
        Gain.resize(Nel);
    }
}

void SigProc1D::interpolate_line(const line<float> &input_data, line<float> &output_data)
{
    if ( input_data.size() == 1 )
    {
        for (int i=0 ; i<output_data.size() ; i++) { output_data.data[i] = input_data.data[0];}
    }
    else if ( input_data.size() <= output_data.size() )
    {
        int nmin = 0, nmax = 0;
        float delta_x = 0.f, gradient = 0.f;;
        nmin = input_data.size();
        nmax = output_data.size();
        line<float> input_x(nmin), output_x(nmax);
        for (int i=0 ; i<nmax ; i++)
        {
            output_x.data[i] = float(i);
        }
        input_x.data[0] = output_x.data[0];
        input_x.data[nmin-1] = output_x.data[nmax-1];
        delta_x = (input_x.data[nmin-1] - input_x.data[0]) / (float(nmin-1));
        for (int i=1 ; i<nmin-1 ; i++)
        {
            input_x.data[i] = input_x.data[i-1] + delta_x;
        }
        int id_xmin=0, id_xmax=1;
        output_data.data[0] = input_data.data[0];
        gradient = (input_data.data[1] - input_data.data[0]) / delta_x;
        while (id_xmax < nmax-1)
        {
            if ( input_x.data[id_xmin] == output_x.data[id_xmax] ) 
            {
                output_data.data[id_xmax] = input_data.data[id_xmin];
                id_xmax++;
            }
            else
            {
                output_data.data[id_xmax] = input_data.data[id_xmin] + gradient * (output_x.data[id_xmax] - input_x.data[id_xmin]);
                id_xmax++;
            }
            if (id_xmin+2 < nmin)
            {
                if (output_x.data[id_xmax] >= input_x.data[id_xmin+1]) 
                {
                    id_xmin++;
                    gradient = (input_data.data[id_xmin+1] - input_data.data[id_xmin]) / delta_x;
                }
            }
        }
        output_data.data[nmax-1] = input_data.data[nmin-1];
    }
    else
    {
        int nmin = 0, nmax = 0;
        float delta_x = 0.f, gradient = 0.f;
        nmin = output_data.size();
        nmax = input_data.size();
        line<float> input_x(nmax), output_x(nmin);
        for (int i=0 ; i<nmin ; i++)
        {
            output_x.data[i] = float(i);
        }
        input_x.data[0] = output_x.data[0];
        input_x.data[nmax-1] = output_x.data[nmin-1];
        delta_x = (input_x.data[nmax-1] - input_x.data[0]) / (float(nmax-1));
        for (int i=1 ; i<nmax-1 ; i++)
        {
            input_x.data[i] = input_x.data[i-1] + delta_x;
        }
        int id_xmin=1, id_xmax=1;
        output_data.data[0] = input_data.data[0];
        while (id_xmin < nmin-1)
        {
            if ( input_x.data[id_xmax] == output_x.data[id_xmin] )
            {
                output_data.data[id_xmin] = input_data.data[id_xmax];
                id_xmin++;
                id_xmax++;
            }
            else
            {
                while (input_x.data[id_xmax+1] < output_x.data[id_xmin])
                {
                    id_xmax++;
                }
                if (input_x.data[id_xmax] == output_x.data[id_xmin])
                {
                    output_data.data[id_xmin] = input_data.data[id_xmax];
                    id_xmin++;
                    id_xmax++;
                }
                else
                {
                    gradient = (input_data.data[id_xmax+1] - input_data.data[id_xmax]) / delta_x;
                    output_data.data[id_xmin] = input_data.data[id_xmax] + gradient * (output_x.data[id_xmin] - input_x.data[id_xmax]);
                    id_xmin++;
                }
            }
        }
        output_data.data[nmin-1] = input_data.data[nmax-1];
    }
}

void SigProc1D::interpolate_line(const float *input_data, const int input_size, float *output_data, const int output_size)
{
    if ( input_size == 1 )
    {
        for (int i=0 ; i<output_size ; i++) { output_data[i] = input_data[0];}
    }
    else if ( input_size <= output_size )
    {
        int nmin = 0, nmax = 0;
        float delta_x = 0.f, gradient = 0.f;;
        nmin = input_size;
        nmax = output_size;
        line<float> input_x(nmin), output_x(nmax);
        for (int i=0 ; i<nmax ; i++)
        {
            output_x.data[i] = float(i);
        }
        input_x.data[0] = output_x.data[0];
        input_x.data[nmin-1] = output_x.data[nmax-1];
        delta_x = (input_x.data[nmin-1] - input_x.data[0]) / (float(nmin-1));
        for (int i=1 ; i<nmin-1 ; i++)
        {
            input_x.data[i] = input_x.data[i-1] + delta_x;
        }
        int id_xmin=0, id_xmax=1;
        output_data[0] = input_data[0];
        gradient = (input_data[1] - input_data[0]) / delta_x;
        while (id_xmax < nmax-1)
        {
            if ( input_x.data[id_xmin] == output_x.data[id_xmax] ) 
            {
                output_data[id_xmax] = input_data[id_xmin];
                id_xmax++;
            }
            else
            {
                output_data[id_xmax] = input_data[id_xmin] + gradient * (output_x.data[id_xmax] - input_x.data[id_xmin]);
                id_xmax++;
            }
            if (id_xmin+2 < nmin)
            {
                if (output_x.data[id_xmax] >= input_x.data[id_xmin+1]) 
                {
                    id_xmin++;
                    gradient = (input_data[id_xmin+1] - input_data[id_xmin]) / delta_x;
                }
            }
        }
        output_data[nmax-1] = input_data[nmin-1];
    }
    else
    {
        int nmin = 0, nmax = 0;
        float delta_x = 0.f, gradient = 0.f;
        nmin = output_size;
        nmax = input_size;
        line<float> input_x(nmax), output_x(nmin);
        for (int i=0 ; i<nmin ; i++)
        {
            output_x.data[i] = float(i);
        }
        input_x.data[0] = output_x.data[0];
        input_x.data[nmax-1] = output_x.data[nmin-1];
        delta_x = (input_x.data[nmax-1] - input_x.data[0]) / (float(nmax-1));
        for (int i=1 ; i<nmax-1 ; i++)
        {
            input_x.data[i] = input_x.data[i-1] + delta_x;
        }
        int id_xmin=1, id_xmax=1;
        output_data[0] = input_data[0];
        while (id_xmin < nmin-1)
        {
            if ( input_x.data[id_xmax] == output_x.data[id_xmin] )
            {
                output_data[id_xmin] = input_data[id_xmax];
                id_xmin++;
                id_xmax++;
            }
            else
            {
                while (input_x.data[id_xmax+1] < output_x.data[id_xmin])
                {
                    id_xmax++;
                }
                if (input_x.data[id_xmax] == output_x.data[id_xmin])
                {
                    output_data[id_xmin] = input_data[id_xmax];
                    id_xmin++;
                    id_xmax++;
                }
                else
                {
                    gradient = (input_data[id_xmax+1] - input_data[id_xmax]) / delta_x;
                    output_data[id_xmin] = input_data[id_xmax] + gradient * (output_x.data[id_xmin] - input_x.data[id_xmax]);
                    id_xmin++;
                }
            }
        }
        output_data[nmin-1] = input_data[nmax-1];
    }
}

void SigProc1D::edit_raw_data_vector(const line<float> &nRaw_data)
{
    int Ninput = nRaw_data.size();
    resize(Ninput);
    Raw_data = nRaw_data;
    Filtered_data = nRaw_data;
}

void SigProc1D::reset_filtered_data_vector()
{
    Filtered_data = Raw_data;
}

line<float> SigProc1D::get_filtered_data()
{
    return Filtered_data;
}

line<float> SigProc1D::get_internal_gain()
{
    return Gain;
}

line<float> SigProc1D::get_gain()
{
    return Input_gain;
}

int SigProc1D::get_internal_vector_size()
{
    if ( (Raw_data.size() == Nel) && (Filtered_data.size() == Nel) && (Gain.size() == Nel) )
    {
        return Nel;
    }
    else
    {
        return 0; // if the size of each vector does not correspond, their is a bug somewhere
    }
}

/*
void SigProc1D::edit_internal_vector_size(int nNel)
{

}
*/

//shift data
void SigProc1D::shift_data(float value)
{
    Filtered_data+value;
}

void SigProc1D::shift_data(const line<float> &input_data, line<float> &output_data, float value)
{
    output_data = input_data;
    output_data + value;
}

void SigProc1D::shift_data(const float *input_data, float *output_data, int data_size, float value)
{
    for (int i=0 ; i<data_size ; i++)
    {
        output_data[i] = input_data[i] + value;
    }
}

//center data around 0
void SigProc1D::center_data()
{
    Filtered_data - Filtered_data.mean();
}

void SigProc1D::center_data(const line<float> &input_data, line<float> &output_data)
{
    output_data = input_data;
    output_data - output_data.mean();
}

void SigProc1D::center_data(const float *input_data, float *output_data, int data_size)
{
    float mean_value = input_data[0];
    for (int i=1 ; i<data_size ; i++) {mean_value += input_data[i];}
    mean_value /= float(data_size);
    for (int i=0 ; i<data_size ; i++) {output_data[i] = input_data[i] - mean_value;}
}

//absolute value
void SigProc1D::absolute_value()
{
    for (int i=0 ; i<Nel ; i++)
    {
        Filtered_data(i) = std::abs(Filtered_data(i));
    }
}

void SigProc1D::absolute_value(const line<float> &input_data, line<float> &output_data)
{
    output_data = input_data;
    for (int i=0 ; i<input_data.size() ; i++)
    {
        output_data.data[i] = std::abs(input_data.data[i]);
    }
}

void SigProc1D::absolute_value(const float *input_data, float *output_data, int data_size)
{
    for (int i=0; i<data_size ; i++)
    {
        if (input_data[i] < 0) {output_data[i] = -input_data[i];}
        else {output_data[i] = input_data[i];}
    }
}

void SigProc1D::normalize()
{
    float fmax = 0.f;
    line<float> ldata;
    absolute_value(Filtered_data, ldata);
    fmax = ldata.max();
    Filtered_data / fmax;
}

void SigProc1D::normalize(const line<float> &input_data, line<float> &output_data)
{
    float fmax = 0.f;
    line<float> ldata;
    absolute_value(input_data, ldata);
    output_data = input_data;
    fmax = ldata.max();
    output_data / fmax;
}

void SigProc1D::normalize(const float *input_data, float *output_data, int data_size)
{
    float fmax = input_data[0];
    float *ldata = new float[data_size]();
    absolute_value(input_data, ldata, data_size);
    for (int i=1 ; i<data_size ; i++)
    {
        if (input_data[i] > fmax) {fmax = input_data[i];}
    }
    for (int i=0 ; i<data_size ; i++) {output_data[i] = input_data[i] / fmax;}
    delete[] ldata;
}

void SigProc1D::get_dB()
{
    for (int i=0 ; i<Nel ; i++)
    {
        Filtered_data.data[i] = 20.f*std::log10(Filtered_data.data[i]);
    }
}

void SigProc1D::get_dB(const line<float> &input_data, line<float> &output_data)
{
    if ( output_data.size() != input_data.size() ) { output_data.resize(input_data.size());}
    for (int i=0 ; i<input_data.size() ; i++)
    {
        output_data.data[i] = 20.f*std::log10(input_data.data[i]);
    }
}

void SigProc1D::get_dB(const float *input_data, float *output_data, int data_size)
{
    for (int i=0 ; i<Nel ; i++)
    {
        output_data[i] = 20.f*std::log10(input_data[i]);
    }
}

void SigProc1D::apply_constant_gain(float value)
{
    Filtered_data * value;
}

void SigProc1D::apply_constant_gain(const line<float> &input_data, line<float> &output_data, float value)
{
    if (output_data.size() != input_data.size()) {output_data.resize(input_data.size());}
    for (int i=0 ; i<input_data.size() ; i++)
    {
        output_data.data[i] = input_data.data[i] * value;
    }
}

void SigProc1D::apply_constant_gain(const float *input_data, float *output_data, int data_size, float value)
{
    for (int i=0 ; i<data_size ; i++)
    {
        output_data[i] = input_data[i] * value;
    }
}

void SigProc1D::edit_internal_gain(const line<float> &new_gain)
{
    Input_gain = new_gain;
    interpolate_line(Input_gain, Gain);
}

void SigProc1D::apply_gain()
{
    Filtered_data = Filtered_data * Gain;
}

void SigProc1D::apply_external_gain(const line<float> &new_gain)
{
    line<float> local_gain(Nel);
    interpolate_line(new_gain, local_gain);
    Filtered_data = Filtered_data * local_gain;
}

void SigProc1D::apply_external_gain(const line<float> &input_data, line<float> &output_data, const line<float> &new_gain)
{
    line<float> local_gain(Nel);
    interpolate_line(new_gain, local_gain);
    output_data = input_data * local_gain;
}

void SigProc1D::apply_external_gain(const float *input_data, float *output_data, int data_size, const float *new_gain, int gain_size)
{
    interpolate_line(new_gain, gain_size, output_data, data_size);
    for (int i=0 ; i<data_size ; i++)
    {
        output_data[i] *= input_data[i];
    }
}
