#ifndef SIGPROC1D_HPP
#define SIGPROC1D_HPP

/*
 * author: Jerome Dubois
 * version: 1.0
 * date: 03/2020
 *
 * class with 1D signal processing
 * can be used with determined size using internal buffers
 * can be used with arbitrary size with input output function
 *
 *
 */

#include"line.hpp"

//TODO make a template class????
//first version as float
class SigProc1D
{
    private:
        int Nel; //number of element for internal buffers
        line<float> Raw_data = line<float>(); //raw data, not changed when applying filter
        line<float> Filtered_data = line<float>(); //data with asked signal(s) processing
        line<float> Gain = line<float>(); //vector to apply 
        line<float> Input_gain = line<float>(); //asked gain that gain have arbitrary size so it can be recall

        //TODO
        //line<float> Output_data for decimation
        //line<float> FIR kernel
        //line<float> IIR kernel...
        
        void resize(int nNel);
        void interpolate_line(const line<float> &input_data, line<float> &output_data);
        void interpolate_line(const float *input_data, const int input_size, float *output_data, const int output_size);
        //void interpolate_gain_vector();

    public:
        SigProc1D(int nNel = 1);
        //~SigProc1D(); //no need destructor yet
        void print();

        void edit_raw_data_vector(const line<float> &nRaw_data);
        void reset_filtered_data_vector(); // reset Filtered data vector to raw data vector
        //TODO add a edit_raw_data_vector(line<float>, int size) to resize SigProc1D to size even if line.size differente ??
        line<float> get_filtered_data();
        line<float> get_internal_gain();
        line<float> get_gain();

        int get_internal_vector_size();
        //void edit_internal_vector_size(int nNel);

        //shift data with value, output = input + value
        void shift_data(float value);
        void shift_data(const line<float> &input_data, line<float> &output_data, float value);
        void shift_data(const float *input_data, float *ouput_data, int data_size, float value);
        //center data (mean data = 0)
        void center_data();
        void center_data(const line<float> &input_data, line<float> &output_data);
        void center_data(const float *input_data, float *output_data, int data_size);
        //get absolute value of the data
        void absolute_value();
        void absolute_value(const line<float> &input_data, line<float> &output_data);
        void absolute_value(const float *input_data, float *output_data, int data_size);
        //normalize the vector
        void normalize();
        void normalize(const line<float> &input_data, line<float> &output_data);
        void normalize(const float *input_data, float *output_data, int data_size);
        //get energie in dB
        void get_dB();
        void get_dB(const line<float> &input_data, line<float> &output_data);
        void get_dB(const float *input_data, float *output_data, int data_size);
        //apply scalar gain to vector
        void apply_constant_gain(float value);
        void apply_constant_gain(const line<float> &input_data, line<float> &output_data, float value);
        void apply_constant_gain(const float *input_data, float *output_data, int data_size, float value);
        //apply internal vector gain
        void edit_internal_gain(const line<float> &ngain);
        void apply_gain(); 
        //apply external vector gain
        void apply_external_gain(const line<float> &ngain);
        void apply_external_gain(const line<float> &input_data, line<float> &output_data, const line<float> &ngain);
        void apply_external_gain(const float *input_data, float *output_data, int data_size, const float *ngain, int gain_size);

        //TODO decimation with decimation type (regular, mean, min, max)
        //TODO FIR
        //TODO IQ
        //TODO FFT
        //TODO IIR 
};

#endif
