#include<iostream>

#include"SigProc2D.hpp"

SigProc2D::SigProc2D()
{
    // init Grayscale
    LUT_change = false;
    edit_LUT_thread_number(1);
    edit_LUT_bound(0.f, 2048.f);
    edit_LUT_gray_level(8);
    edit_LUT_quantification_type(linear);
    init_LUT();

    // init Filter
    edit_filter_thread_number(1);
    edit_filter_type(no_filter);

    // init scan_conv
    scan_conv_change = false;
    edit_scan_conv_thread_number(1);
}

void SigProc2D::edit_LUT_thread_number(int number_of_thread)
{
    LUT.edit_number_of_packet(number_of_thread);
    std::shared_ptr<ThreadPool> lut_pool = std::make_shared<ThreadPool>(number_of_thread);
    LUT.setUp(lut_pool);
}

void SigProc2D::edit_LUT_bound(float nLowerBound, float nUpperBound)
{
    LUT.edit_Bound(nLowerBound, nUpperBound);
    LUT_change = true;
}

void SigProc2D::edit_LUT_gray_level(int nGrayLevel)
{
    LUT.edit_GrayLevel(nGrayLevel);
    LUT_change = true;
}

void SigProc2D::edit_LUT_quantification_type(QuantificationType nType)
{
    LUT.edit_QuantificationType(nType);
    LUT_change = true;
}

void SigProc2D::init_LUT()
{
    LUT.initFilter();
    LUT_change = false;
}

void SigProc2D::apply_LUT(const image<float> &InputImage, image<int> &OutputImage)
{
    int nrow = InputImage.row_size(), ncolumn = InputImage.column_size();
    if (LUT_change)
    {
        init_LUT();
    }
    if ( (nrow != OutputImage.row_size()) || (ncolumn != OutputImage.column_size()) )
    {
        std::cout << "Warning, size of output image for apply LUT is not the same than input image, image is resized" << std::endl;
        OutputImage.resize(nrow, ncolumn);
    }
    LUT.applyFilter(InputImage, OutputImage);
}

float SigProc2D::get_LUT_lower_bound()
{
    return LUT.get_lower_bound();
}

float SigProc2D::get_LUT_upper_bound()
{
    return LUT.get_upper_bound();
}

QuantificationType SigProc2D::get_LUT_quantification_type()
{
    return LUT.get_quantification_type();
}

int SigProc2D::get_LUT_gray_level()
{
    return LUT.get_gray_level();
}

int SigProc2D::get_LUT_number_of_thread()
{
    return LUT.get_number_of_packets();
}

void SigProc2D::print_LUT()
{
    LUT.print();    
}

// Filter
void SigProc2D::edit_filter_thread_number(int number_of_thread)
{
    Filter2D.edit_number_of_packet(number_of_thread);
    std::shared_ptr<ThreadPool> filter_pool = std::make_shared<ThreadPool>(number_of_thread);
    Filter2D.setUp(filter_pool);
}

void SigProc2D::edit_filter_type(FilterType nType)
{
    Filter2D.edit_filter_type(nType);
}

void SigProc2D::edit_filter_kernel(const image<float> &nkernel)
{
    Filter2D.edit_kernel(nkernel);
}

void SigProc2D::edit_filter_kernel(float f00, float f01, float f02, float f10, float f11, float f12, float f20, float f21, float f22)
{
    Filter2D.edit_kernel(f00, f01, f02, f10, f11, f12, f20, f21, f22);
}

void SigProc2D::apply_filter(const image<float> &input, image<float> &output)
{
    if ( (input.row_size() != output.row_size()) || (input.column_size() != output.column_size()) )
    {
        std::cout << "Warning, size of output image is different from input image for filter calculation, resizing to input size." << std::endl;
        output.resize(input.row_size(), input.column_size());
    }
    Filter2D.applyFilter(input, output);
}

void SigProc2D::print_filter()
{
    Filter2D.print();
}

// scan conversion
void SigProc2D::print_scan_conv()
{
    ScanConv.print();
}

void SigProc2D::edit_scan_conv_thread_number(int number_of_thread)
{
    ScanConv.edit_number_of_packet(number_of_thread);
    std::shared_ptr<ThreadPool> sc_pool = std::make_shared<ThreadPool>(number_of_thread);
    ScanConv.setUp(sc_pool);
}

void SigProc2D::edit_scan_conv_image_size(int nNx, int nNy)
{
    ScanConv.edit_image_size(nNx, nNy);
    scan_conv_change = true;
}

void SigProc2D::edit_scan_conv_us_image_parameters(int nNline, int nNpoint)
{
    ScanConv.edit_us_image_parameters(nNline, nNpoint);
    scan_conv_change = true;
}

void SigProc2D::edit_scan_conv_speed_of_sound(float nspeed)
{
    ScanConv.edit_speed(nspeed);
    scan_conv_change = true;
}

void SigProc2D::edit_scan_conv_mechanism(Mechanism nmechanism, float nh0, float ndx, float nrb, float nli, float nti)
{
    ScanConv.edit_mechanism(nmechanism, nh0, ndx, nrb, nli, nti);
    scan_conv_change = true;
}

void SigProc2D::edit_scan_conv_scanning(float nsector, int nNline, ScanningOption nscanning_option)
{
    ScanConv.edit_scanning(nsector, nNline, nscanning_option);
    scan_conv_change = true;
}

void SigProc2D::edit_scan_conv_depth(float nR0, float nRf)
{
    ScanConv.edit_depth(nR0, nRf);
    scan_conv_change = true;
}

void SigProc2D::get_scan_conv_max_depth(float &lRmin, float &lRmax)
{
    ScanConv.get_maximum_depth(lRmin, lRmax);
}

void SigProc2D::calcul_scan_conv_kernel()
{
    ScanConv.kernel_calculation();
    scan_conv_change = false;
}

void SigProc2D::edit_scan_conv_sampling(float nsampling_rate, int ndecimation, float nspeed, int nNpoint)
{
    ScanConv.edit_sampling_us(nsampling_rate, ndecimation, nspeed, nNpoint);
    scan_conv_change = true;
}

void SigProc2D::edit_scan_conv_delay_us(float ndelay)
{
    ScanConv.edit_delay_us(ndelay);
    scan_conv_change = true;
}

void SigProc2D::edit_scan_conv_sampling_mm(float nsampling_mm, int nNpoint)
{
    ScanConv.edit_sampling_mm(nsampling_mm, nNpoint);
    scan_conv_change = true;
}

void SigProc2D::edit_scan_conv_delay_mm(float ndelay_mm)
{
    ScanConv.edit_delay_mm(ndelay_mm);
    scan_conv_change = true;
}

void SigProc2D::apply_scan_conv(const image<float> &input, image<float> &output)
{
    if ( (input.row_size() != ScanConv.getNline()) || (input.column_size() != ScanConv.getNpoint()) )
    {
        std::cout << "Warning, size of input image for scan conversion is not good, size is (" << input.row_size() << "," << input.column_size() << "), while (" << ScanConv.getNpoint() << "," << ScanConv.getNline() << ") is expected. No calculation made" << std::endl;
        return;
    }
    if ( (output.row_size() != ScanConv.getNx()) || (output.column_size() != ScanConv.getNy()) )
    {
        std::cout << "Warning, size of output image for scan conversion is not good, automatic resize to the good one." << std::endl;
        output.resize(ScanConv.getNx(), ScanConv.getNy());
    }
    if (scan_conv_change)
    {
        calcul_scan_conv_kernel();
    }
    ScanConv.image_scan_conversion(input, output);
}

void SigProc2D::scan_conv_change_image_background(image<float> &input, float background)
{
    if ( (input.row_size() != ScanConv.getNx()) || (input.column_size() != ScanConv.getNy()) )
    {
        std::cout << "Warning, size of output image for changing background is not good, automatic resize to the good one." << std::endl;
        input.resize(ScanConv.getNx(), ScanConv.getNy());
    }
    if (scan_conv_change)
    {
        calcul_scan_conv_kernel();
    }
    ScanConv.change_image_background(input, background);
}
void SigProc2D::scan_conv_change_image_value(image<float> &input, float value)
{
    if ( (input.row_size() != ScanConv.getNx()) || (input.column_size() != ScanConv.getNy()) )
    {
        std::cout << "Warning, size of output image for changing image value is not good, automatic resize to the good one." << std::endl;
        input.resize(ScanConv.getNx(), ScanConv.getNy());
    }
    if (scan_conv_change)
    {
        calcul_scan_conv_kernel();
    }
    ScanConv.change_image_value(input, value);

}
