#ifndef SIGPROC2D_HPP
#define SIGPROC2D_HPP

#include"image.hpp"
#include"GrayScale.hpp"
#include"Filter.hpp"
#include"scan_conversion_float.hpp"

/*
 * author: Jerome Dubois
 * version: 1.0
 * date: 04/2020
 *
 * class with 2D signal processing
 *
 *
 *
 *
 *
 */

class SigProc2D
{
    private:
        GrayScale LUT = GrayScale();
        bool LUT_change;
        Filter Filter2D = Filter();
        scan_conv ScanConv = scan_conv();
        bool scan_conv_change;

    public:
        SigProc2D();
        ~SigProc2D(){}
        void print();

        // LUT function
        void print_LUT();
        void edit_LUT_thread_number(int number_of_thread);
        void edit_LUT_bound(float nLowerBound, float nUpperBound);
        void edit_LUT_gray_level(int nGrayLevel);
        void edit_LUT_quantification_type(QuantificationType nType);
        void init_LUT();
        void apply_LUT(const image<float> &InputImage, image<int> &OutputImage);

        float get_LUT_lower_bound();
        float get_LUT_upper_bound();
        QuantificationType get_LUT_quantification_type();
        int get_LUT_gray_level();
        int get_LUT_number_of_thread();


        // Image filter function
        void print_filter();
        void edit_filter_thread_number(int number_of_thread);
        void edit_filter_type(FilterType nType);
        void edit_filter_kernel(const image<float> &nKernel);
        void edit_filter_kernel(float f00, float f01, float f02, float f10, float f11, float f12, float f20, float f21, float f22);
        void apply_filter(const image<float> &input, image<float> &output);

        FilterType get_filter_type() {return Filter2D.get_filter_type();}
        image<float> get_filter_kernel() {return Filter2D.get_kernel();}
        int get_filter_number_of_thread() {return Filter2D.get_number_of_packet();}


        // Scan conversion function
        void print_scan_conv();
        void edit_scan_conv_thread_number(int number_of_thread);
            // general settings
        void edit_scan_conv_image_size(int nNx, int nNy);
        void edit_scan_conv_us_image_parameters(int nNline, int nNpoint);
        void edit_scan_conv_speed_of_sound(float nspeed);
        void edit_scan_conv_mechanism(Mechanism nmechanism, float nh0=0.f, float ndx=0.f, float nrb=0.f, float nli=0.f, float nti=0.f);
        void set_scan_conv_image_square() {ScanConv.set_image_square();} // deltax = deltay but the image not fill the whole x or y
        void set_scan_conv_image_arbitrary() {ScanConv.set_image_arbitrary();} // deltax and deltay are made such the scan converted image fill the whole space 
        void edit_scan_conv_scanning(float nsector, int nNline, ScanningOption nscanning_option = constant_theta);
        void edit_scan_conv_depth(float nR0, float nRf);
        void get_scan_conv_max_depth(float &lRmin, float &lRmax);
        void calcul_scan_conv_kernel();
            // settings using ultrasound card parameters
        void edit_scan_conv_sampling(float nsampling_rate, int ndecimation, float nspeed, int nNpoint);
        void edit_scan_conv_delay_us(float ndelay);
            // settings using mm parameters
        void edit_scan_conv_sampling_mm(float nsampling_mm, int nNpoint);
        void edit_scan_conv_delay_mm(float ndelay_mm);
            // output image
        void apply_scan_conv(const image<float> &input, image<float> &output);
        void scan_conv_change_image_background(image<float> &input, float background);
        void scan_conv_change_image_value(image<float> &input, float value);

        int get_scan_conv_Nx() {return ScanConv.getNx();}
        int get_scan_conv_Ny() {return ScanConv.getNy();}
        int get_scan_conv_Nline() {return ScanConv.getNline();}
        int get_scan_conv_Npoint() {return ScanConv.getNpoint();}
        int get_scan_conv_number_of_thred() {return ScanConv.get_number_of_packet();}
        float get_scan_conv_speed() {return ScanConv.get_speed();}
        Mechanism get_scan_conv_mechanism() {return ScanConv.get_mechanism();}
        float get_scan_conv_h0() {return ScanConv.get_h0();}
        float get_scan_conv_dx() {return ScanConv.get_dx();}
        float get_scan_conv_rb() {return ScanConv.get_rb();}
        float get_scan_conv_li() {return ScanConv.get_li();}
        float get_scan_conv_ti() {return ScanConv.get_ti();}
        float get_scan_conv_sector() {return ScanConv.get_sector();}
        ScanningOption get_scan_conv_scanning_option() {return ScanConv.get_scanning_option();}
        bool get_scan_conv_square_image() {return ScanConv.is_square();}
        float get_scan_conv_R0() {return ScanConv.get_R0();}
        float get_scan_conv_Rf() {return ScanConv.get_Rf();}
        float get_scan_conv_sampling_rate() {return ScanConv.get_sampling_rate();}
        int get_scan_conv_decimation() {return ScanConv.get_decimation();}
        float get_scan_conv_delay_us() {return ScanConv.get_delay_us();}
        float get_scan_conv_delay_mm() {return ScanConv.get_delay_mm();}
};

#endif
