#ifndef VECTOR_HPP
#define VECTOR_HPP 

/*
 *  contributors:   - Clement Le Couedic
 *                  - Jerome Dubois
 *  version: 1.2
 *  date: 03/2020
 *
 *
 *
 */

#include<iostream>
#include<cstring> //for memcpy
#include<fstream>
#include"pointer_management.hpp"

//Forward declaration
template <class T> class line;
template <class T> std::ostream& operator<<(std::ostream &oOS, const line<T>& nline);

//class declaration
template <class T>
class line
{
  private:
    int Nel;
    bool verbose;

  public:    
    T *data;
    
    line(int nNel=1); //constructor
    line(const line<T> &other); //copy constructor
    line& operator=(const line<T> &other); //copy assignement operator
    line(line<T> &&other); //move constructor
    line& operator=(line<T> &&other); //move assignement operator
    ~line();

    //operator
    T operator() (int pos) const; 
    T& operator() (int pos);
    void operator+ (T const value);
    void operator- (T const value);
    void operator* (T const value);
    void operator/ (T const value);
    //line<T> operator+ (line<T> const input, T const value);
    //TODO operator scalar + and -?
    //TODO operator scalar * and /?
    friend std::ostream& operator << <> (std::ostream &oOS, const line<T>& nline);

    //various function
    inline int size() const {return int(Nel);};
    void resize(int nNel);
    void reset();
    T sum();
    T min();
    T max();
    T mean();
    void write(std::string);
    void load(std::string);
    
    //
    inline void set_verbose_on() {verbose = true;}
    inline void set_verbose_off() {verbose = false;}
    inline bool get_verbose() const {return verbose;} 
};

//Constructor
template <class T>
line<T>::line(int nNel)
{
    Nel=nNel;
    create_vector(data, Nel);
    set_verbose_off();
}

//Copy constructor
template <class T>
line<T>::line(const line<T> &other)
{
    Nel = other.Nel;
    verbose = other.verbose;
    
    //vector
    create_vector(data, Nel);
    std::memcpy(data, other.data, Nel*sizeof(T));
}

//Copy assignement operator
template <class T>
line<T>& line<T>::operator=(const line<T> &other)
{
    if (this!=&other)
    {
        if (Nel != other.Nel)
        {
            Nel = other.Nel;
            verbose = other.verbose;
            resize_vector(data, Nel);
        }
        std::memcpy(data, other.data, Nel*sizeof(T));
    }
    return *this;
}

//Move constructor
template <class T>
line<T>::line(line<T> &&other)
{
    Nel = other.Nel;
    verbose = other.verbose;
    //delete_vector(data);
    data = other.data;
    
    //release other ressources
    other.Nel = 0;
    other.verbose = false;
    other.data = nullptr;
}

//Move assignement operator
template <class T>
line<T>& line<T>::operator=(line<T> &&other)
{
    if (this != &other)
    {
        Nel = other.Nel;
        verbose = other.verbose;
        delete_vector(data);
        data = other.data;
        
        //release other line ressources
        other.Nel = 0;
        other.verbose = false;
        other.data = nullptr;
    }
    return *this;

}

//Destructor
template <class T>
line<T>::~line()
{
    Nel = 0;
    verbose = false;
    delete_vector(data);
}

//Operators
template<class T>
T line<T>::operator() (int pos) const
{
    if(pos < 0)
    {
        if (verbose)
        {
            std::cout << "Warning, index out of bound, trying to access negative index: " << pos << std::endl;
        }
        return data[0];
    }
    else if (pos >= Nel)
    {
        if (verbose)
        {
            std::cout << "Warning, index out of bound, trying to access element: " << pos+1 << " (ID " << pos << ") whereas number of element is: " << Nel << std::endl;
        }
        return data[Nel-1];
    }
    else
    {
        return data[pos];
    }
}

template<class T>
T& line<T>::operator() (int pos)
{
    if(pos < 0)
    {
        if (verbose)
        {
            std::cout << "Warning, index out of bound, trying to access negative index: " << pos << std::endl;
        }
        return data[0];
    }
    else if (pos >= Nel)
    {
        if (verbose)
        {
            std::cout << "Warning, index out of bound, trying to access element: " << pos+1 << " (ID " << ") whereas number of element is: " << Nel << std::endl;
        }
        return data[Nel-1];
    }
    else
    {
        return data[pos];
    }
}

template<class T>
void line<T>::operator+ (T const value)
{
    for (int i=0 ; i<Nel ; i++) {data[i] += value;}
}

template<class T>
void line<T>::operator- (T const value)
{
    for (int i=0 ; i<Nel ; i++) {data[i] -= value;}
}

template<class T>
void line<T>::operator* (T const value)
{
    for (int i=0 ; i<Nel ; i++) {data[i] *= value;}
}

template<class T>
void line<T>::operator/ (T const value)
{
    for (int i=0 ; i<Nel ; i++) {data[i] /= value;}
}

template<class T>
line<T> operator+ (line<T> const &linput, line<T> const &rinput)
{
    if (linput.size() != rinput.size())
    {
        std::cout << "Warning, using sum line<T> a + line<T> b, size of both line is not equal, returning a" << std::endl;
        return linput;
    }
    else
    {
        line<T> output = line<T>(linput.size());
        for (int i=0 ; i<linput.size() ; i++)
        {
            output.data[i] = linput.data[i] + rinput.data[i];
        }
        return output;
    }
}

template<class T>
line<T> operator- (line<T> const &linput, line<T> const &rinput)
{
    if (linput.size() != rinput.size())
    {
        std::cout << "Warning, using substract line<T> a - line<T> b, size of both line is not equal, returning a" << std::endl;
        return linput;
    }
    else
    {
        line<T> output = line<T>(linput.size());
        for (int i=0 ; i<linput.size() ; i++)
        {
            output.data[i] = linput.data[i] - rinput.data[i];
        }
        return output;
    }
}

template<class T>
line<T> operator* (line<T> const &linput, line<T> const &rinput)
{
    if (linput.size() != rinput.size())
    {
        std::cout << "Warning, using multiplication line<T> a * line<T> b, size of both line is not equal, returning a" << std::endl;
        return linput;
    }
    else
    {
        line<T> output = line<T>(linput.size());
        for (int i=0 ; i<linput.size() ; i++)
        {
            output.data[i] = linput.data[i] * rinput.data[i];
        }
        return output;
    }
}

template<class T>
line<T> operator/ (line<T> const &linput, line<T> const &rinput)
{
    if (linput.size() != rinput.size())
    {
        std::cout << "Warning, using division line<T> a / line<T> b, size of both line is not equal, returning a" << std::endl;
        return linput;
    }
    else
    {
        line<T> output = line<T>(linput.size());
        for (int i=0 ; i<linput.size() ; i++)
        {
            output.data[i] = linput.data[i] / rinput.data[i];
        }
        return output;
    }
}



template<class T>
std::ostream& operator<<(std::ostream &oOS, const line<T>& nline)
{
    oOS << "line Nelement =" << nline.Nel << ";" <<std::endl;
    for (int i=0 ; i<nline.Nel ; i++) {oOS << " " << nline.data[i];}
    oOS << std::endl;
    return oOS;
}

//General functions
template <class T>
void line<T>::resize(int nNel)
{
    if (nNel != Nel)
    {
        Nel=nNel;
        resize_vector(data,Nel);
    }
}

template <class T>
void line<T>::reset()
{
    for (int i=0 ; i<Nel ; i++) {data[i]=T(0);}
}

template <class T>
T line<T>::sum()
{
    T element_sum = data[0];
    for (int i=1 ; i<Nel ; i++) {element_sum += data[i];}
    return element_sum;
}

template <class T>
T line<T>::min()
{
    T min_value = data[0];
    for (int i=1 ; i<Nel ; i++)
    {
        if (data[i] < min_value) {min_value = data[i];}
    }
    return min_value;
}

template <class T>
T line<T>::max()
{
    T max_value = data[0];
    for (int i=1 ; i<Nel ; i++)
    {
        if (data[i] > max_value) {max_value = data[i];}
    }
    return max_value;
}

template <class T>
T line<T>::mean()
{
    double mean_value = 0.f;
    mean_value = double(sum());
    mean_value /= double(Nel);
    return T(mean_value);
}

template <class T>
void line<T>::write(std::string name)
{
    std::ofstream txtfile;
    txtfile.open(name);
    if (!txtfile)
    {
        std::cout << "Error opening file " << name << " to write line<T>" << std::endl;
        exit(1);
    }

    txtfile << Nel << std::endl;
    for (int i=0 ; i<Nel ; i++) {txtfile << data[i] << std::endl;}
    txtfile.close();
}

template <class T>
void line<T>::load(std::string name)
{
    std::ifstream txtfile;
    txtfile.open(name);
    if (!txtfile)
    {
        std::cout << "Error opening file " << name << " to load line<T>" << std::endl;
        exit(1);
    }

    int nNel=1;
    txtfile >> nNel;
    if (nNel != Nel) {resize(nNel);}
    for (int i=0 ; i<Nel ; i++) {txtfile >> data[i];}
    txtfile.close();
}

#endif
