#ifndef POINTER_MANAGEMENT_HPP
#define POINTER_MANAGEMENT_HPP

#include<new>
#include<cstdint>
#include<string>
#include<fstream>
#include<iostream>

/*---
Librairy: pointer_management
author: Jérôme Dubois
date: 01/19
version: 1.1

utility: simple usage of pointer as vector or matrix of any type

update:
- all elements of vectors and matrices are now initialized at 0 
---*/

/*-----
Prototypes
-----*/
template <typename T> void create_vector (T *&vec, const unsigned int N);
template <typename T> void resize_vector (T *&vec, const unsigned int Nnew);
template <typename T> void delete_vector (T *&vec);
template <typename T> void create_matrix (T **&mat, const unsigned int Nrow, const unsigned int Ncolumn);
template <typename T> void resize_matrix (T **&mat, const unsigned int Nrow_old, const unsigned int Ncolumn_old, const unsigned int Nrow, const unsigned int Ncolumn);
template <typename T> void delete_matrix (T **&mat, const unsigned int Nrow);

/*-----
Template
------*/
//Vector
template <typename T> void create_vector (T *&vec, const unsigned int N)
{
  vec=new T[N]();
}

template <typename T> void resize_vector (T *&vec, const unsigned int Nnew)
{
  if (vec==nullptr) {create_vector(vec, Nnew);}
  else
  {
    delete_vector(vec);
    vec=new T[Nnew]();
  }
}

template <typename T> void delete_vector (T *&vec)
{
  if (vec!=nullptr)
  {
    delete[] vec;
    vec=nullptr;
  }
}

//Matrix
template <typename T> void create_matrix (T **&mat, const unsigned int Nrow, const unsigned int Ncolumn)
{
  mat = new T*[Nrow];
  for (unsigned int i=0 ; i<Nrow ; i++) {create_vector(mat[i], Ncolumn);}
}

template <typename T> void resize_matrix (T **&mat, const unsigned int Nrow_old, const unsigned int Ncolumn_old, const unsigned int Nrow, const unsigned int Ncolumn)
{
  if (mat==nullptr) {create_matrix(mat, Nrow, Ncolumn);}
  else if(Nrow_old==Nrow)
  {
    if (Ncolumn!=Ncolumn_old)
    {
      for (unsigned int i=0 ; i<Nrow ; i++) {resize_vector(mat[i], Ncolumn);}
    }
  }
  else
  {
    for (unsigned int i=0 ; i<Nrow_old ; i++) {delete_vector(mat[i]);}
    delete[] mat;
    create_matrix(mat, Nrow, Ncolumn);
  }
}

template <typename T> void delete_matrix (T **&mat, const unsigned int Nrow)
{
  if (mat!=nullptr)
  {
    for (unsigned int i=0 ; i<Nrow ; i++) {delete_vector(mat[i]);}
    delete[] mat;
    mat=nullptr;
  }
}

#endif
