#include<iostream>
#include<cmath>

#include"scan_conversion_float.hpp"

scan_conv::scan_conv(int nNline, int nNpoint, int nNx, int nNy)
{
    Nline = nNline;
    Npoint = nNpoint;
    Nx = nNx;
    Ny = nNy;

    edit_number_of_packet(1);
    square = true;
    error = 0;
    interpolation_type = bilinear;
    R0 = 0;
    Rf = 200;
    Rmin = 0;
    Rmax = 200;

    mechanism = pure_rotation;
    scanning_option = constant_theta;
    h0 = 0.f;
    //hp = 0.f;
    sector = 60.f;
    sector_rd = sector*M_PI/180.f; 
    total_sector = 360.f;
    total_sector_rd = 2*M_PI;
    dx = 0.f;
    rb = 0.f;
    li = 0.f;
    ti = 0.f;
    ti_rd = 0.f;
    rtang = 0.f;
    x0 = 0.f;

    decimation = 1;
    sampling_rate = 80.f;
    speed = 1540.f;
    edit_sampling_us(sampling_rate, decimation, speed, Npoint);
    delay_us = 0.f;
    delay_mm = 0.f;

    N_point_to_change=1;
    N_out=1;

    //vectors
    create_vector(x_vector, Nx);
    create_vector(y_vector, Ny);
    create_vector(xi, Nline);
    create_vector(yi, Nline);
    create_vector(ri, Nline);
    create_vector(thetai, Nline);

    //matrix
    create_matrix(Image, Nx, Ny);
    create_matrix(indicial_x_y_tmp, 2, Nx*Ny);
    create_matrix(indicial_x_y_out_tmp, 2, Nx*Ny);
    //create_matrix(indicial_x_y, 2, N_point_to_change); //N_point_to_change unknow actually
    //create_matrix(indicial_x_y_out, 2, N_out);
    create_matrix(indicial_r_theta, 4, N_point_to_change);
    create_matrix(weight, 4, N_point_to_change);
    create_matrix(r_vector, 1, Npoint);
    create_matrix(theta_vector, Nline, 1);
    create_matrix(r_matrix, Nx, Ny);
    create_matrix(theta_matrix, Nx, Ny);
}

scan_conv::~scan_conv()
{
    //vectors
    delete_vector(x_vector);
    delete_vector(y_vector);
    delete_vector(xi);
    delete_vector(yi);
    delete_vector(ri);
    delete_vector(thetai);

    //matrix
    delete_matrix(Image, Nx);
    delete_matrix(indicial_x_y_tmp, 2);
    delete_matrix(indicial_x_y_out_tmp, 2);
    //delete_matrix(indicial_x_y, 2); //N_point_to_change unknow actually
    //delete_matrix(indicial_x_y_out, 2);
    delete_matrix(indicial_r_theta, 4);
    delete_matrix(weight, 4);
    if (mechanism == pure_rotation) {delete_matrix(r_vector, 1);}
    else {delete_matrix(r_vector, Nline);}
    delete_matrix(theta_vector, Nline);
    delete_matrix(r_matrix, Nx);
    delete_matrix(theta_matrix, Nx);
}

void scan_conv::setUp(std::shared_ptr<ThreadPool> iThreadPool)
{
    mThreadPool = iThreadPool;
}

void scan_conv::print()
{
    std::cout << "scan conversion parameters:" << std::endl;

    std::cout << "Nline = " << Nline << std::endl;
    std::cout << "Npoint = " << Npoint << std::endl;
    std::cout << "Nx = " << Nx << std::endl;
    std::cout << "Ny = " << Ny << std::endl;

    std::cout << "error = " <<error << std::endl;
    std::cout << "interpolation_type: ";
    switch (interpolation_type)
    {
        case bilinear:
            std::cout << "bilinear interpolation";
            break;
        case distance_weight:
            std::cout << "distance/weight interpolation";
            break;
    }
    std::cout << std::endl;
    std::cout << "R0 = " << R0 << std::endl;
    std::cout << "Rf = " << Rf << std::endl;
    std::cout << "Rmin = " << Rmin << std::endl;
    std::cout << "Rmax = " << Rmax << std::endl;

    std::cout << "mechanism type: ";
    switch (mechanism)
    {
        case pure_rotation:
            std::cout << "pure rotation";
            break;
        case translation_rotation:
            std::cout << "linear displacement chart";
            break;
        case lounger:
            std::cout << "lounger";
    }
    std::cout << std::endl;
    std::cout << "scanning option: ";
    switch (scanning_option)
    {
        case constant_elevation:
            std::cout << "delat x constant (equivalent to constant deplacement of chart)";
            break;
        case constant_theta:
            std::cout << "delta theta constant";
            break;
    }
    std::cout << std::endl;
    std::cout << "rb = " << rb << std::endl;
    //std::cout << "hp = " << hp << std::endl;
    std::cout << "sector (degree) = " << sector << std::endl;
    std::cout << "sector (rad) = " << sector_rd << std::endl;
    std::cout << "total sector (degree) = " << total_sector << std::endl;
    std::cout << "total sector (rad) = " << total_sector_rd << std::endl;
    std::cout << "dx (total displacement of the chart) = " << dx << std::endl;
    std::cout << "h0 = " << h0 << std::endl;
    std::cout << "li = " << li << std::endl;
    std::cout << "ti = " << ti << std::endl;
    std::cout << "ti_rd = " << ti_rd << " rad" << std::endl;
    std::cout << "rtang = " << rtang << std::endl;
    std::cout << "x0 = " << x0 << std::endl;

    std::cout << "decimation = " << decimation << std::endl;
    std::cout << "sampling rate = "  << sampling_rate << " MHz" << std::endl;
    std::cout << "true sampling rate = " << true_sampling_rate << " MHz" << std::endl;
    std::cout << "speed of sound = " << speed << " m.s^{-1}" << std::endl;
    std::cout << "dr (sampling depth) = " << dr << " mm"<< std::endl;
    std::cout << "delay (us) = " << delay_us << std::endl;
    std::cout << "delay (mm) = " << delay_mm << std::endl;

    std::cout << "N point to change = " << N_point_to_change << std::endl;
    std::cout << "N out = " << N_out << std::endl;
}

void scan_conv::print_position()
{
    std::cout << "xi: ";
    for (int i=0 ; i<Nline ; i++) {std::cout << xi[i] << " ";}
    std::cout << std::endl;

    std::cout << "yi: ";
    for (int i=0 ; i<Nline ; i++) {std::cout << yi[i] << " ";}
    std::cout << std::endl;

    std::cout << "ri: ";
    for (int i=0 ; i<Nline ; i++) {std::cout << ri[i] << " ";}
    std::cout << std::endl;

    std::cout << "thetai: ";
    for (int i=0 ; i<Nline ; i++) {std::cout << thetai[i] << " ";}
    std::cout << std::endl;

    std::cout << "dthetai: ";
    for (int i=0 ; i<Nline-1 ; i++) {std::cout << thetai[i+1]-thetai[i] << " ";}
    std::cout << std::endl;

}

void scan_conv::edit_Npoint(int nNpoint)
{
    if (nNpoint!=Npoint)
    {
        //vectors
        //resize_vector(xi, Nline);
        //resize_vector(yi, Nline);
        //resize_vector(ri, Nline);
        //resize_vector(thetai, Nline);

        //matrix
        switch(mechanism)
        {
            case pure_rotation:
                resize_matrix(r_vector, 1, Npoint, 1, nNpoint);
                break;
            default:
                resize_matrix(r_vector, Nline, Npoint, Nline, nNpoint);
                break;
        }
        Npoint = nNpoint;
    }
}

void scan_conv::edit_Nline(int nNline)
{
    if (nNline!=Nline)
    {
        //vectors
        resize_vector(xi, nNline);
        resize_vector(yi, nNline);
        resize_vector(ri, nNline);
        resize_vector(thetai, nNline);

        //matrix
        if (mechanism != pure_rotation)
        {    
            resize_matrix(r_vector, Nline, Npoint, nNline, Npoint);
        }
        resize_matrix(theta_vector, Nline, 1, nNline, 1);
        Nline=nNline;
    }
}

void scan_conv::edit_sampling_us(float nsampling_rate, int ndecimation, float nspeed, int nNpoint)
{
    sampling_rate = nsampling_rate;
    decimation = ndecimation;
    true_sampling_rate = sampling_rate/(float(decimation));
    speed = nspeed;
    dr = speed/(2.f*true_sampling_rate*1000.f);
    edit_Npoint(nNpoint);
}

void scan_conv::edit_sampling_mm(float sampling_mm, int nNpoint)
{
    true_sampling_rate = speed / (2.f*dr*1000.f);
    decimation = int(sampling_rate / true_sampling_rate);
    true_sampling_rate = sampling_rate/(float(decimation));
    dr = sampling_mm;
    std::cout << "true sampling depth is " << speed/(2.f*true_sampling_rate*1000.f) << " mm ???" << std::endl;
    edit_Npoint(nNpoint);
}

void scan_conv::edit_mechanism(Mechanism nmechanism, float nrb, float ndx, float nh0, float nli, float nti)
{
    if ( (nmechanism < 0) || (nmechanism >2) )
    {
        std::cout << "Warning mechanism must be in [0;2], taking default value 0 (pure rotation)" << std::endl;
        nmechanism = pure_rotation;
    }
    switch (nmechanism)
    {
        case pure_rotation:
            rb = nrb;
            //hp = nhp;
            dx = 0.f;
            h0 = 0.f;
            li = 0.f;
            ti = 0.f;
            ti_rd = 0.f;
            rtang = 0.f;
            x0 = 0.f;
            if (mechanism != pure_rotation) {resize_matrix(r_vector, Nline, Npoint, 1, Npoint);}
            mechanism = nmechanism;
            break;
        case translation_rotation:
            if ((nh0 == 0.f) || (ndx == 0.f)) {std::cout << "Warning h0 and dx can't be equal to 0 for linear displacement, no changement of mechanism" << std::endl;}
            else
            {
                rb = nrb;
                //hp = nhp;
                dx = ndx;
                h0 = 0.f;
                li = 0.f;
                ti = 0.f;
                ti_rd = 0.f;
                rtang = 0.f;
                x0 = 0.f;
                if (mechanism == pure_rotation) {resize_matrix(r_vector, 1, Npoint, Nline, Npoint);}
                mechanism = nmechanism;
            }
            break;
        case lounger:
            if ((nh0 == 0.f) || (ndx == 0.f)) {std::cout << "Warning h0 and dx can't be equal to 0 for lounger, no changement of mechanism" << std::endl;}
            else
            {
                rb = nrb;
                //hp = nhp;
                dx = ndx;
                h0 = nh0;
                li = nli;
                ti = nti;
                ti_rd = ti*M_PI/180.f;
                rtang = (li - rb*std::tan(ti_rd))*std::cos(ti_rd);
                x0 = 0.f;
                if (mechanism == pure_rotation) {resize_matrix(r_vector, 1, Npoint, Nline, Npoint);}
                mechanism = nmechanism;
            }
            break;
    }
}

void scan_conv::edit_scanning(float nsector, int nNline, ScanningOption nscanning_option)
{
    scanning_option=nscanning_option;
    edit_Nline(nNline);

    if (nsector < 0.f)
    {
        std::cout << "Warning, sector can't be negative, we take " << -nsector << " instead of " << nsector << std::endl;
        nsector = -nsector;
    }
    else if (nsector == 0.f)
    {
        std::cout << "Warning, sector can't be equal to zero, we take 60° as default sector" << " instead of " << nsector << std::endl;
        nsector = 60.f;
    }
    if (nsector > 360.f)
    {
        float oldsector = nsector;
        nsector = fmod(nsector,360.f); //modulo 360.f
        std::cout << "Warning, sector can't be superior to one turn, taking modulo 360 value: " << nsector << " instead of " << oldsector << std::endl;
    }

    switch (mechanism)
    {
        case pure_rotation:
            total_sector = 360.f;
            break;
        case translation_rotation:
            total_sector_rd = 2.f*std::atan(dx/2.f/h0);
            total_sector = total_sector_rd*180.f/M_PI;
            break;
        case lounger:
            total_sector_rd = std::atan(dx/2.f/h0);
            total_sector_rd -= std::abs(ti_rd);
            total_sector_rd *= 2.f;
            total_sector = total_sector_rd*180.f/M_PI;
            break;
        default:
            std::cout << "Warning, mechanism type " << mechanism << "not implement yet" << std::endl;
    }
    if (nsector > total_sector)
    {
        nsector = total_sector;
        std::cout << "Warning, sector exceed total sector swept by the mechanism, take maximum value: " << nsector << std::endl;
    }
    sector = nsector;
    sector_rd = sector*M_PI/180.f;
    if (mechanism == lounger) {x0 = rtang/std::cos(sector_rd/2.f);}
}

void scan_conv::edit_delay_us(float ndelay)
{
    delay_us = ndelay;
    delay_mm = speed*delay_us/1000.f/2.f; //divide by 2 for back and forth
}

void scan_conv::edit_delay_mm(float ndelay)
{
    delay_mm = ndelay;
    delay_us = delay_mm/speed*1000.f*2.f;
}

void scan_conv::calcul_maximum_depth()
{
    std::cout << "(scan::conv::calcul_maximum_depth()) Scan conversion edit depth, asked: Rmin = " << Rmin << ", Rmax = " << Rmax << " and dr = " << dr << std::endl;

    switch (mechanism)
    {
        case pure_rotation:
            Rmin = delay_mm + rb;
            if (Rmin < 0.f) {Rmin = 0.f;}
            Rmax = delay_mm + float(Npoint-1)*dr + rb;
            break;
        case translation_rotation:
        case lounger:
            //Rmin = delay_mm + vector_min(ri);
            Rmin = delay_mm + vector_max(ri);
            if (Rmin < 0.f) {Rmin = 0.f;}
            //Rmax = delay_mm + float(Npoint-1)*dr + vector_max(ri);
            Rmax = delay_mm + float(Npoint-1)*dr + vector_min(ri);
            break;
    }
}

void scan_conv::calcul_transducer_position()
{
    int i=0;
    float dthetai=0.f;
    float dxi=0.f;
    float dxt=0.f;
    float true_theta=0.f; //true theta of the lounger, shift due to transducer angle
    
    switch (mechanism)
    {
        case pure_rotation:
            if (scanning_option == constant_theta) //dtheta constant
            {
                dthetai = sector_rd/float(Nline-1);
                thetai[0]=-sector_rd/2.f;
                thetai[Nline-1]=sector_rd/2.f;
                for (i=1 ; i<Nline-1 ; i++) {thetai[i] = thetai[i-1] + dthetai;}

                //std::cout << "thetai calculation: ";
                //for (i=0 ; i<Nline-1 ; i++) {std::cout << thetai[i] << " ";}
                std::cout << std::endl;
            }
            if (scanning_option == constant_elevation) //dx constant
            {
                dxt = 2.f*std::tan(sector_rd/2.f); //fictive transducer at 1 mm on a line
                dxi = dxt/float(Nline-1);
                dxt = -std::tan(sector_rd/2.f);
                thetai[0]=-sector_rd/2.f;
                thetai[Nline-1]=sector_rd/2.f;
                for (i=1 ; i<Nline-1 ; i++)
                {
                    dxt += dxi;
                    thetai[i] = std::atan(dxt);
                }
            }
            for (i=0 ; i<Nline ; i++)
            {
                ri[i] = rb;
                xi[i] = -rb*std::sin(thetai[i]);
                yi[i] = rb*std::cos(thetai[i]);
            }
            break;
        case translation_rotation:
            if (scanning_option == constant_theta) //dtheta constant
            {
                dthetai = sector_rd/float(Nline-1);
                thetai[0]=-sector_rd/2.f;
                thetai[Nline-1]=sector_rd/2.f;
                for (i=1 ; i<Nline-1 ; i++) {thetai[i] = thetai[i-1] + dthetai;}
            }
            if (scanning_option == constant_elevation) //dx constant
            {
                dxt = 2.f*h0*std::tan(sector_rd/2.f);
                dxi = dxt/float(Nline-1);
                dxt = h0*std::tan(-sector_rd/2.f);
                thetai[0]=-sector_rd/2.f;
                thetai[Nline-1]=sector_rd/2.f;
                for (i=1 ; i<Nline-1 ; i++)
                {
                    dxt += dxi;
                    thetai[i] = std::atan(-dxt/h0);
                }
            }
            for (i=0 ; i<Nline ; i++)
            {
                xi[i] = -rb*std::tan(thetai[i]);
                yi[i] = rb;
                ri[i] = std::sqrt(xi[i]*xi[i] + yi[i]*yi[i]);
            }
            break;
        case lounger:
            //determination of the angle of the shot lines
            if (scanning_option == constant_theta) //dtheta constant
            {
                dthetai = sector_rd/float(Nline-1);
                thetai[0]=-sector_rd/2.f;
                thetai[Nline-1]=sector_rd/2.f;
                for (i=1 ; i<Nline-1 ; i++) {thetai[i] = thetai[i-1] + dthetai;}
            }
            if (scanning_option == constant_elevation) //dx constant
            {
                //not working yet
                //dxt = 2.f*h0*std::tan(sector_rd/2.f);
                //dxi = dxt/float(Nline-1);
                //dxt = std::tan(-sector_rd/2.f);
                thetai[0]=-sector_rd/2.f-ti_rd; //true initial theta
                thetai[Nline-1]=sector_rd/2.f-ti_rd; //true final theta
                dxt = h0*(std::tan(thetai[Nline-1])-std::tan(thetai[0]));
                dxi = dxt/float(Nline-1);
                dxt = h0*std::tan(thetai[0]);
                for (i=1 ; i<Nline-1 ; i++)
                {
                    dxt += dxi;
                    thetai[i] = std::atan(dxt/h0);
                }
            }
            //follow have to be rechecked
            //true position of the transducer
            for (i=0 ; i<Nline ; i++)
            {
                // true position of the transducers depend on theta of the lounger, not the angle of the shot lines
                true_theta = thetai[i]-ti_rd;
                xi[i] = -rb*std::sin(true_theta) + li*std::cos(true_theta);
                yi[i] = rb*std::cos(true_theta) + li*std::sin(true_theta);
            }
            //position of the transducer in the model space
            for (i=0 ; i<Nline ; i++)
            {
                // in the model space we consider theta of the lines, not theta of the lounger
                //thetai[i] += ti_rd; //theta in the model space
                xi[i] -= rtang/std::cos(thetai[i]);
                ri[i] = rb/std::abs(rb)*std::sqrt(xi[i]*xi[i] + yi[i]*yi[i]);
            }
            break;
    }
}

float scan_conv::vector_min(float *in)
{
    float min=in[0];
    for (int i=1 ; i<Nline ; i++)
    {
        if (in[i] < min) {min=in[i];}
    }
    return min;
}

float scan_conv::vector_max(float *in)
{
    float max=in[0];
    for (int i=1 ; i<Nline ; i++)
    {
        if (in[i] > max) {max=in[i];}
    }
    return max;
}

void scan_conv::edit_depth(float nR0, float nRf)
{
    calcul_transducer_position();
    calcul_maximum_depth();
    if ( (nRf < Rmin) || (nR0 > Rmax) )
    {
        std::cout << "Warning, you asked for a scan from " << nR0 << " to " << nRf << "whereas limit scan depth is from " << Rmin << " to " << Rmax << "... Verify your settings, limit scan depth is taken for calculation." << std::endl;
        nR0 = Rmin*1.001f;
        nRf = Rmax=0.999f;
    }
    if (nR0 < Rmin)
    {
        std::cout << "Warning, R0 is toos small, taking " << Rmin << std::endl;
        nR0 = Rmin*1.001f;
    }
    R0 = nR0;

    if (nRf < nR0)
    {
        std::cout << "Warning, Rf can't be smaller than R0, taking " << Rmin+10.f << std::endl;
        nRf=R0+10.f;
    }
    if (nRf > Rmax)
    {
        std::cout << "Warning, Rf is too high, taking " << Rmax << std::endl;
        nRf = Rmax*0.999f;
    }
    Rf = nRf;
}

void scan_conv::get_maximum_depth(float &lRmin, float &lRmax)
{
    calcul_transducer_position();
    calcul_maximum_depth();
    lRmin = Rmin;
    lRmax = Rmax;
}

void scan_conv::edit_us_image_parameters(int nNline, int nNpoint)
{
    edit_Npoint(nNpoint);
    edit_Nline(nNline);
    edit_scanning(sector, Nline, scanning_option);
    edit_depth(R0, Rf);
}

void scan_conv::edit_image_size(int nNx, int nNy)
{
    //vectors
    std::cout << "resize vector" << std::endl;
    resize_vector(x_vector, nNx);
    resize_vector(y_vector, nNy);

    //matrix
    std::cout << "resize matrix" << std::endl;
    resize_matrix(Image, Nx, Ny, nNx, nNy);
    resize_matrix(indicial_x_y_tmp, 2, Nx*Ny, 2, nNx*nNy);
    resize_matrix(indicial_x_y_out_tmp, 2, Nx*Ny, 2, nNx*nNy);
    resize_matrix(r_matrix, Nx, Ny, nNx, nNy);
    resize_matrix(theta_matrix, Nx, Ny, nNx, nNy);
    Nx = nNx;
    Ny = nNy;
}

void scan_conv::fill_xy_vector()
{
	int i=0;
	float half_sec=sector_rd/2.f;
	float x0=-Rf*std::sin(half_sec);
	float xf= -x0;
	//float delta_x=(2.f*xf)/(float(Nx-1));
    float delta_x = (2.f*xf)/(float(Nx)); //position is center of the pixel so no N-1
	float y0=R0*std::cos(half_sec);
	float yf=Rf;
	//float delta_y=(yf-y0)/(float(Ny-1));
    float delta_y = (yf-y0)/(float(Ny));

    if (square)
    {
        if (delta_x < delta_y)
        {
            delta_x = delta_y;
            int N_mean = Nx/2;
            x0 = -(float(N_mean)-0.5f)*delta_x;
            xf = -x0;
        }
        else
        {
            delta_y = delta_x;
            float y_mean = (yf+y0)/2;
            int N_mean = Ny/2;
            y0 = y_mean-(float(N_mean)-0.5f)*delta_y;
            yf = y_mean+(float(N_mean)+0.5f)*delta_y;
        }
    }

	x_vector[0]=x0;
	y_vector[0]=y0;
	if (Nx <= Ny)
	{
		for (i=1 ; i<Nx-1 ; i++)
		{
			x_vector[i]=x_vector[i-1]+delta_x;
			y_vector[i]=y_vector[i-1]+delta_y;
		}
		for (i=Nx-1 ; i<Ny-1 ; i++)
		{
			y_vector[i]=y_vector[i-1]+delta_y;
		}
	}
	else
	{
		for (i=1 ; i<Ny-1 ; i++)
		{
			x_vector[i]=x_vector[i-1]+delta_x;
			y_vector[i]=y_vector[i-1]+delta_y;
		}
		for (i=Ny-1 ; i<Nx-1 ; i++)
		{
			x_vector[i]=x_vector[i-1]+delta_x;
		}
	}
	x_vector[Nx-1]=xf;
	y_vector[Ny-1]=yf;
}

void scan_conv::fill_rtheta_vectors()
{
	int i=0, j=0;
	//float delta_theta=sector_rd/(float(Nline-1));
	//float half_sec=sector_rd/2.f;

	switch (mechanism)
	{
		case pure_rotation: 
			r_vector[0][0] = delay_mm+rb;
			if (r_vector[0][0]>R0) 
            {
                std::cout << "Warning R0 out of bound, too small, min = " << Rmin << std::endl;
                error=1;
            }
			for (i=1 ; i<Npoint ; i++) {r_vector[0][i]=r_vector[0][i-1]+dr;}
			if (r_vector[0][Npoint-1]<Rf) 
            {
                std::cout << "Warning Rf out of bound, too high, max = " << Rmax << std::endl;
                error=2;
            }
			for (i=0 ; i<Nline ; i++) {theta_vector[i][0]=thetai[i];}
			break;

		case translation_rotation:
		    for (j=0 ; j<Nline ; j++)
		    {
				r_vector[j][0]=delay_mm+ri[j];
				if (r_vector[j][0]>R0) 
                {
                    std::cout << "Warning R0 out of bound, too small for line " << j << ", min = " << Rmin << std::endl;
                    error=1;
                }
				for (i=1 ; i<Npoint ; i++) {r_vector[j][i]=r_vector[j][i-1]+dr;}
				if (r_vector[j][Npoint-1]<Rf) 
                {
                    std::cout << "Warning Rf out of bound, too high for line " << j << ", max= " << Rmax << std::endl;
                    error=2;
                }

				for (i=0 ; i<Nline ; i++) {theta_vector[i][0]=thetai[i];}
		    }
			break;

		case lounger: //identical to case 1
		    for (j=0 ; j<Nline ; j++)
		    {
				r_vector[j][0]=delay_mm+ri[j];
				if (r_vector[j][0]>R0) 
                {
                    std::cout << "Warning R0 out of bound, too small for line " << j << ", min = " << Rmin << std::endl;
                    error=1;
                }
				for (i=1 ; i<Npoint ; i++) {r_vector[j][i]=r_vector[j][i-1]+dr;}
				if (r_vector[j][Npoint-1]<Rf) 
                {
                    std::cout << "Warning Rf out of bound, too high for line " << j << ", max= " << Rmax << std::endl;
                    error=2;
                }

				for (i=0 ; i<Nline ; i++) {theta_vector[i][0]=thetai[i];}
		    }
		    break;
        
        default :
            std::cout << "Warning, MechanismType undefined" << std::endl;
			break;
	}
}

void scan_conv::fill_rtheta_matrices()
{
	int i=0, j=0, N_tmp=0, N_out_tmp=0;
	float half_sec=sector_rd/2.f;
    
    
    if (mechanism == lounger) //special test for mecanic version 3, due to local deformation, may be delete after tests because we won't image at this position where local effects appears, change of r and theta must not be cleared!!!!
    {
        float x=0.f, y=0.f;
        for (j=0 ; j<Ny ; j++)
        {
            for (i=0 ; i<Nx ; i++)
            {
                r_matrix[i][j]=std::sqrt(x_vector[i]*x_vector[i]+y_vector[j]*y_vector[j]);
                theta_matrix[i][j]=-std::atan(x_vector[i]/y_vector[j]);
                if ((r_matrix[i][j]>R0) && (r_matrix[i][j]<Rf) && (theta_matrix[i][j]>-half_sec) && (theta_matrix[i][j]<half_sec))
                {
                    indicial_x_y_tmp[0][N_tmp]=i;
                    indicial_x_y_tmp[1][N_tmp]=j;
                    N_tmp++;
                }
                else
                {
                    x=r_matrix[i][j]*std::cos(theta_matrix[i][j]);
                    y=r_matrix[i][j]*std::cos(theta_matrix[i][j]);
                    if (((x+x0)*(x+x0)+y*y > rtang*rtang*1.01) && (std::abs(y) < 2.f) && (li/std::abs(li)*x <= 0.f) && (std::abs(x) < std::abs(x0))) // useful only if we look close to crossing point, very local exception points
                    {
                        indicial_x_y_tmp[0][N_tmp]=i;
                        indicial_x_y_tmp[1][N_tmp]=j;
                        N_tmp++; 
                    }
                    else
                    {
                        indicial_x_y_out_tmp[0][N_out_tmp]=i;
                        indicial_x_y_out_tmp[1][N_out_tmp]=j;
                        N_out_tmp++;
                    }
                }
            }
        }
        
        float t1=0.f, t2=0.f, theta1=0.f, theta2=0.f, theta=0.f;
        float dr=rtang, x0l=x0;
        
        //change real grid for model grid
        for (j=0 ; j<Ny ; j++)
        {
            for (i=0 ; i<Nx ; i++)
            {
                y=r_matrix[i][j]*std::cos(theta_matrix[i][j]);
                x=r_matrix[i][j]*std::sin(theta_matrix[i][j]);
                t1=(y+std::sqrt(x*x+y*y-dr*dr+x0l*x0l+2.f*x0l*x))/(-dr-x-x0l);
                t2=(y-std::sqrt(x*x+y*y-dr*dr+x0l*x0l+2.f*x0l*x))/(-dr-x-x0l);
                theta1=2.f*std::atan(t1);
                theta2=2.f*std::atan(t2);
                if (std::abs(theta1) <= (sector_rd/2.f))
                {
                    if(std::abs(theta2) <= (sector_rd/2.f))
                    {
                        if (li<0)
                        {
                            if (theta1<theta2) {theta=theta1;}
                            else {theta=theta2;}
                        }
                        else
                        {
                            if (theta1<theta2) {theta=theta2;}
                            else {theta=theta1;}
                        }
                    }
                    else {theta=theta1;}
                    
                    x+=x0l-dr/std::cos(theta);
                    r_matrix[i][j]=std::sqrt(x*x+y*y);
                    theta_matrix[i][j]=std::atan(x/y);
                }
                else if(std::abs(theta2) <= (sector_rd/2.f)) 
                {
                    theta=theta2;
                    
                    x-=x0l-dr/std::cos(theta);
                    r_matrix[i][j]=std::sqrt(x*x+y*y);
                    theta_matrix[i][j]=std::atan(x/y);
                }
                else 
                {
                    r_matrix[i][j]=0.f;
                    theta_matrix[i][j]=0.f;
                }   
            }
        }
    }
    else
    {
        for (j=0 ; j<Ny ; j++)
        {
            for (i=0 ; i<Nx ; i++)
            {
                r_matrix[i][j]=std::sqrt(x_vector[i]*x_vector[i]+y_vector[j]*y_vector[j]);
                theta_matrix[i][j]=std::atan(x_vector[i]/y_vector[j]);
                if (r_matrix[i][j]<R0 || r_matrix[i][j]>Rf || theta_matrix[i][j]<-half_sec || theta_matrix[i][j]>half_sec)
                {
                    indicial_x_y_out_tmp[0][N_out_tmp]=i;
                    indicial_x_y_out_tmp[1][N_out_tmp]=j;
                    N_out_tmp++;
                }
                else
                {
                    indicial_x_y_tmp[0][N_tmp]=i;
                    indicial_x_y_tmp[1][N_tmp]=j;
                    N_tmp++;
                }
            }
        }
    }

    resize_matrix(indicial_r_theta, 4, N_point_to_change, 4, N_tmp);
    resize_matrix(weight, 4, N_point_to_change, 4, N_tmp);
	
    N_point_to_change=N_tmp;
	N_out=Nx*Ny-N_tmp;
}

void scan_conv::indicial_matrix_calculation()
{
	int i=0, k=0, l=0;
	int k1=0, k2=0;
	float r_old=0.f, r_new=0.f, theta_old=0.f, theta_new=0.f;

    std::cout << std::endl << std::endl;
    std::cout << "mechanism: " << mechanism << std::endl << std::endl;

	switch(mechanism)
	{
		case pure_rotation:
			for (i=0 ; i<N_point_to_change ; i++)
			{
				r_new=r_matrix[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]];
				theta_new=theta_matrix[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]];
				if (r_new > r_old) //r increase and decrease continuously
				{
					while ((r_vector[0][k+1] < r_new) && (k < Npoint-1)) {k++;}
				}
				else if (r_new<r_old)
				{
					while ((r_vector[0][k] > r_new) && (k > 0)) {k--;}
				}
				r_old=r_new;
				if (theta_new < theta_old) //theta increase continuously but decrease from theta to -theta
				{
					l=0;
					while ((theta_vector[l+1][0] < theta_new) && (l < Nline-1)) {l++;}
				}
				else if (theta_new > theta_old)
				{
					while ((theta_vector[l+1][0] < theta_new) && (l < Nline-1)) {l++;}
				}
				theta_old=theta_new;

				indicial_r_theta[0][i]=k;
				indicial_r_theta[1][i]=k+1;
				indicial_r_theta[2][i]=l;
				indicial_r_theta[3][i]=l+1;
			}
			break;
		case translation_rotation:
        case lounger:
			for (i=0 ; i<N_point_to_change ; i++)
			{
				r_new=r_matrix[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]];
				theta_new=theta_matrix[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]];

				if (theta_new < theta_old) //theta increase continuously but decrease from theta to -theta
				{
					l=0;
					while ((theta_vector[l+1][0] < theta_new) && (l < Nline-1)) {l++;}
				}
				else if (theta_new > theta_old)
				{
					while ((theta_vector[l+1][0] < theta_new) && (l < Nline-1)) {l++;}
				}
				
				//search of r[k] for line l
				if (r_vector[l][k1] < r_new)
				{
					while ((r_vector[l][k1+1] < r_new) && (k1 < Npoint-1)) {k1++;}
				}
				else if (r_vector[l][k1] > r_new)
				{
					while ((r_vector[l][k1] > r_new) && (k1 > 0)) {k1--;}
				}
				//search of r[k] for line l+1
				if (r_vector[l+1][k2] < r_new)
				{
					while ((r_vector[l+1][k2+1] < r_new) && (k2 < Npoint-1)) {k2++;}
				}
				else if (r_vector[l+1][k2] > r_new)
				{
					while ((r_vector[l+1][k2] > r_new) && (k2 > 0)) {k2--;}
				}

				theta_old=theta_new;

				indicial_r_theta[0][i]=k1;
				indicial_r_theta[1][i]=k2;
				indicial_r_theta[2][i]=l;
				indicial_r_theta[3][i]=l+1;
			}
			break;
	}
}

void scan_conv::weight_matrix_calculation()
{
	int i=0;
	float rl=0.f, tl=0.f, r1=0.f, r2=0.f, t1=0.f, t2=0.f, P1=0.f, P2=0.f, P3=0.f, P4=0.f;
	float Rk1=0.f, Rk2=0.f, Psi=0.f;

	if (interpolation_type == bilinear) //linear interpolation
	{
		for (i=0 ; i< N_point_to_change ; i++)
		{
			switch (mechanism)
			{
				case pure_rotation:
					rl=r_matrix[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]];
					tl=theta_matrix[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]];
					r1=r_vector[0][indicial_r_theta[0][i]];
					r2=r_vector[0][indicial_r_theta[1][i]];
					t1=theta_vector[indicial_r_theta[2][i]][0];
					t2=theta_vector[indicial_r_theta[3][i]][0];
					P1=(r2-rl)/(r2-r1);
					P2=(rl-r1)/(r2-r1);
					P3=(t2-tl)/(t2-t1);
					P4=(tl-t1)/(t2-t1);
					weight[0][i]=P1*P3;
					weight[1][i]=P1*P4;
					weight[2][i]=P2*P3;
					weight[3][i]=P2*P4;
					break;
				case translation_rotation:
                case lounger:
					rl=r_matrix[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]];
					tl=theta_matrix[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]];
					r1=r_vector[indicial_r_theta[2][i]][indicial_r_theta[0][i]];
					r2=r_vector[indicial_r_theta[3][i]][indicial_r_theta[1][i]];
					t1=theta_vector[indicial_r_theta[2][i]][0];
					t2=theta_vector[indicial_r_theta[3][i]][0];
					Rk1=(rl-r1)/(dr);
					Rk2=(rl-r2)/(dr);
					Psi=(tl-t1)/(t2-t1);
					weight[0][i]=(1.f-Psi)*(1.f-Rk1);
					weight[1][i]=Psi*(1.f-Rk2);
					weight[2][i]=(1.f-Psi)*Rk1;
					weight[3][i]=Psi*Rk2;
					break;
				default:
                    std::cout << "Warning, MechanismType undefined" << std::endl;
					break;
			}
		}
	}
	else if (interpolation_type == distance_weight) //distance interpolation
	{
		float r3=0.f, r4=0.f;
		float xl=0.f, xt=0.f, yl=0.f, yt=0.f;
		float norm=0.f;
		switch (mechanism)
		{
			case pure_rotation:	
				for (i=0 ; i<N_point_to_change ; i++)
				{
					xl=x_vector[indicial_x_y_tmp[0][i]];
					yl=y_vector[indicial_x_y_tmp[1][i]];
					xt=r_vector[0][indicial_r_theta[0][i]]*std::sin(theta_vector[indicial_r_theta[2][i]][0]);
					yt=r_vector[0][indicial_r_theta[0][i]]*std::cos(theta_vector[indicial_r_theta[2][i]][0]);
					r1=std::sqrt((xl-xt)*(xl-xt)+(yl-yt)*(yl-yt));
					xt=r_vector[0][indicial_r_theta[0][i]]*std::sin(theta_vector[indicial_r_theta[3][i]][0]);
      				yt=r_vector[0][indicial_r_theta[0][i]]*std::cos(theta_vector[indicial_r_theta[3][i]][0]);
      				r2=std::sqrt((xl-xt)*(xl-xt)+(yl-yt)*(yl-yt));
					xt=r_vector[0][indicial_r_theta[1][i]]*std::sin(theta_vector[indicial_r_theta[2][i]][0]);
      				yt=r_vector[0][indicial_r_theta[1][i]]*std::cos(theta_vector[indicial_r_theta[2][i]][0]);
      				r3=std::sqrt((xl-xt)*(xl-xt)+(yl-yt)*(yl-yt));
					xt=r_vector[0][indicial_r_theta[1][i]]*std::sin(theta_vector[indicial_r_theta[3][i]][0]);
      				yt=r_vector[0][indicial_r_theta[1][i]]*std::cos(theta_vector[indicial_r_theta[3][i]][0]);
      				r4=std::sqrt((xl-xt)*(xl-xt)+(yl-yt)*(yl-yt));
					norm=1/r1+1/r2+1/r3+1/r4;
					norm=1/norm;
					weight[0][i]=norm/r1;
					weight[1][i]=norm/r2;
					weight[2][i]=norm/r3;
					weight[3][i]=norm/r4;
				}
				break;
			case translation_rotation:
            case lounger: //identical to case 1
				for (i=0 ; i<N_point_to_change ; i++)
				{
					xl=x_vector[indicial_x_y_tmp[0][i]];
					yl=y_vector[indicial_x_y_tmp[1][i]];
					xt=r_vector[indicial_r_theta[2][i]][indicial_r_theta[0][i]]*std::sin(theta_vector[indicial_r_theta[2][i]][0]);
					yt=r_vector[indicial_r_theta[2][i]][indicial_r_theta[0][i]]*std::cos(theta_vector[indicial_r_theta[2][i]][0]);
					r1=std::sqrt((xl-xt)*(xl-xt)+(yl-yt)*(yl-yt));
					xt=r_vector[indicial_r_theta[3][i]][indicial_r_theta[1][i]]*std::sin(theta_vector[indicial_r_theta[3][i]][0]);
      				yt=r_vector[indicial_r_theta[3][i]][indicial_r_theta[1][i]]*std::cos(theta_vector[indicial_r_theta[3][i]][0]);
      				r2=std::sqrt((xl-xt)*(xl-xt)+(yl-yt)*(yl-yt));
					xt=r_vector[indicial_r_theta[2][i]][indicial_r_theta[0][i]+1]*std::sin(theta_vector[indicial_r_theta[2][i]][0]);
      				yt=r_vector[indicial_r_theta[2][i]][indicial_r_theta[0][i]+1]*std::cos(theta_vector[indicial_r_theta[2][i]][0]);
      				r3=std::sqrt((xl-xt)*(xl-xt)+(yl-yt)*(yl-yt));
					xt=r_vector[indicial_r_theta[3][i]][indicial_r_theta[1][i]+1]*std::sin(theta_vector[indicial_r_theta[3][i]][0]);
      				yt=r_vector[indicial_r_theta[3][i]][indicial_r_theta[1][i]+1]*std::cos(theta_vector[indicial_r_theta[3][i]][0]);
      				r4=std::sqrt((xl-xt)*(xl-xt)+(yl-yt)*(yl-yt));
					norm=1/r1+1/r2+1/r3+1/r4;
					norm=1/norm;
					weight[0][i]=norm/r1;
					weight[1][i]=norm/r2;
					weight[2][i]=norm/r3;
					weight[3][i]=norm/r4;
				}
				break;
			default:
                std::cout << "Warning, MechanismType undefined" << std::endl;
				break;
		}
	}
}

void scan_conv::kernel_calculation()
{
    calcul_maximum_depth();
    edit_scanning(sector, Nline, scanning_option);
    edit_depth(R0, Rf); //recalculate the different parameters in case the user forgot

    fill_xy_vector();
    fill_rtheta_vectors();
    fill_rtheta_matrices();
    indicial_matrix_calculation();
    weight_matrix_calculation();
}

void scan_conv::image_scan_conversion_raw (float **image_r_theta)
{
	int i=0;
	float P1=0.f, P2=0.f, P3=0.f, P4=0.f;
	for (i=0 ; i<N_point_to_change ; i++)
	{
		switch (mechanism)
		{
			case pure_rotation:
				P1=weight[0][i]*image_r_theta[indicial_r_theta[2][i]][indicial_r_theta[0][i]];
				P2=weight[1][i]*image_r_theta[indicial_r_theta[3][i]][indicial_r_theta[0][i]];
				P3=weight[2][i]*image_r_theta[indicial_r_theta[2][i]][indicial_r_theta[1][i]];
				P4=weight[3][i]*image_r_theta[indicial_r_theta[3][i]][indicial_r_theta[1][i]];
				break;
			case translation_rotation:
			case lounger:
				P1=weight[0][i]*image_r_theta[indicial_r_theta[2][i]][indicial_r_theta[0][i]];
				P2=weight[1][i]*image_r_theta[indicial_r_theta[3][i]][indicial_r_theta[1][i]];
				P3=weight[2][i]*image_r_theta[indicial_r_theta[2][i]][indicial_r_theta[0][i]+1];
				P4=weight[3][i]*image_r_theta[indicial_r_theta[3][i]][indicial_r_theta[1][i]+1];
				break;
			default:
                std::cout << "Warning, MechanismType undefined" << std::endl;
				break;
		}
		Image[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]]=(P1+P2+P3+P4);
	}
}

void scan_conv::image_scan_conversion_raw (const image<float> &input, image<float> &output)
{
	int i=0;
	float P1=0.f, P2=0.f, P3=0.f, P4=0.f;
	for (i=0 ; i<N_point_to_change ; i++)
	{
		switch (mechanism)
		{
			case pure_rotation:
				P1=weight[0][i]*input.data[indicial_r_theta[2][i]][indicial_r_theta[0][i]];
				P2=weight[1][i]*input.data[indicial_r_theta[3][i]][indicial_r_theta[0][i]];
				P3=weight[2][i]*input.data[indicial_r_theta[2][i]][indicial_r_theta[1][i]];
				P4=weight[3][i]*input.data[indicial_r_theta[3][i]][indicial_r_theta[1][i]];
				break;
			case translation_rotation:
			case lounger:
				P1=weight[0][i]*input.data[indicial_r_theta[2][i]][indicial_r_theta[0][i]];
				P2=weight[1][i]*input.data[indicial_r_theta[3][i]][indicial_r_theta[1][i]];
				P3=weight[2][i]*input.data[indicial_r_theta[2][i]][indicial_r_theta[0][i]+1];
				P4=weight[3][i]*input.data[indicial_r_theta[3][i]][indicial_r_theta[1][i]+1];
				break;
			default:
                std::cout << "Warning, MechanismType undefined" << std::endl;
				break;
		}
		output.data[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]]=(P1+P2+P3+P4);
	}
}

void scan_conv::partial_scan_conversion (float **image_r_theta, int start_point, int end_point)
{
	int i=0;
	float P1=0.f, P2=0.f, P3=0.f, P4=0.f;

    if (start_point < 0) {start_point = 0;}
    if (end_point > (N_point_to_change-1)) {end_point = N_point_to_change -1;}

	for (i=start_point ; i<=end_point ; i++)
	{
		switch (mechanism)
		{
			case pure_rotation:
				P1=weight[0][i]*image_r_theta[indicial_r_theta[2][i]][indicial_r_theta[0][i]];
				P2=weight[1][i]*image_r_theta[indicial_r_theta[3][i]][indicial_r_theta[0][i]];
				P3=weight[2][i]*image_r_theta[indicial_r_theta[2][i]][indicial_r_theta[1][i]];
				P4=weight[3][i]*image_r_theta[indicial_r_theta[3][i]][indicial_r_theta[1][i]];
				break;
			case translation_rotation:
			case lounger:
				P1=weight[0][i]*image_r_theta[indicial_r_theta[2][i]][indicial_r_theta[0][i]];
				P2=weight[1][i]*image_r_theta[indicial_r_theta[3][i]][indicial_r_theta[1][i]];
				P3=weight[2][i]*image_r_theta[indicial_r_theta[2][i]][indicial_r_theta[0][i]+1];
				P4=weight[3][i]*image_r_theta[indicial_r_theta[3][i]][indicial_r_theta[1][i]+1];
				break;
			default:
                std::cout << "Warning, MechanismType undefined" << std::endl;
				break;
		}
		Image[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]]=(P1+P2+P3+P4);
	}
}

void scan_conv::partial_scan_conversion (const image<float> &input, image<float> &output, int start_point, int end_point)
{
	int i=0;
	float P1=0.f, P2=0.f, P3=0.f, P4=0.f;

    if (start_point < 0) {start_point = 0;}
    if (end_point > (N_point_to_change-1)) {end_point = N_point_to_change -1;}

	for (i=start_point ; i<=end_point ; i++)
	{
		switch (mechanism)
		{
			case pure_rotation:
				P1=weight[0][i]*input.data[indicial_r_theta[2][i]][indicial_r_theta[0][i]];
				P2=weight[1][i]*input.data[indicial_r_theta[3][i]][indicial_r_theta[0][i]];
				P3=weight[2][i]*input.data[indicial_r_theta[2][i]][indicial_r_theta[1][i]];
				P4=weight[3][i]*input.data[indicial_r_theta[3][i]][indicial_r_theta[1][i]];
				break;
			case translation_rotation:
			case lounger:
				P1=weight[0][i]*input.data[indicial_r_theta[2][i]][indicial_r_theta[0][i]];
				P2=weight[1][i]*input.data[indicial_r_theta[3][i]][indicial_r_theta[1][i]];
				P3=weight[2][i]*input.data[indicial_r_theta[2][i]][indicial_r_theta[0][i]+1];
				P4=weight[3][i]*input.data[indicial_r_theta[3][i]][indicial_r_theta[1][i]+1];
				break;
			default:
                std::cout << "Warning, MechanismType undefined" << std::endl;
				break;
		}
		output.data[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]]=(P1+P2+P3+P4);
	}
}

void scan_conv::image_scan_conversion (float **image_r_theta)
{
    std::vector< std::future<bool> > lResults;
    auto partial_scan = [this, &image_r_theta](int nstart_point, int nend_point)
    {
        partial_scan_conversion(image_r_theta, nstart_point, nend_point);
        return true;
    };

    int lPacketSize = N_point_to_change / NumberOfPackets ;
    int istart_point=0, iend_point=0, i=0;
    int diff = N_point_to_change - lPacketSize*NumberOfPackets;

    iend_point = lPacketSize + diff-1;
    lResults.emplace_back(mThreadPool->enqueue(partial_scan, istart_point, iend_point));
    istart_point = iend_point+1;

    for (i=1 ; i<NumberOfPackets ; i++)
    {
        iend_point += lPacketSize;
        lResults.emplace_back(mThreadPool->enqueue(partial_scan, istart_point, iend_point));
        istart_point = iend_point+1;
    }
    for (auto && lResults: lResults) {lResults.wait();}
}

void scan_conv::image_scan_conversion (const image<float> &input, image<float> &output)
{
    std::vector< std::future<bool> > lResults;
    auto partial_scan = [this, &input, &output](int nstart_point, int nend_point)
    {
        partial_scan_conversion(input, output, nstart_point, nend_point);
        return true;
    };

    int lPacketSize = N_point_to_change / NumberOfPackets ;
    int istart_point=0, iend_point=0, i=0;
    int diff = N_point_to_change - lPacketSize*NumberOfPackets;

    iend_point = lPacketSize + diff-1;
    lResults.emplace_back(mThreadPool->enqueue(partial_scan, istart_point, iend_point));
    istart_point = iend_point+1;

    for (i=1 ; i<NumberOfPackets ; i++)
    {
        iend_point += lPacketSize;
        lResults.emplace_back(mThreadPool->enqueue(partial_scan, istart_point, iend_point));
        istart_point = iend_point+1;
    }
    for (auto && lResults: lResults) {lResults.wait();}
}

void scan_conv::change_image_background (float background)
{
	int i=0;
	for (i=0 ; i<N_out ; i++)
	{
		Image[indicial_x_y_out_tmp[0][i]][indicial_x_y_out_tmp[1][i]]=background;
	}
}

void scan_conv::change_image_background (image<float> &input, float background)
{
	int i=0;
	for (i=0 ; i<N_out ; i++)
	{
		input.data[indicial_x_y_out_tmp[0][i]][indicial_x_y_out_tmp[1][i]]=background;
	}
}

void scan_conv::change_image_background (const image<float> &input, image<float> &output, float background)
{
    output = input;
	int i=0;
	for (i=0 ; i<N_out ; i++)
	{
		output.data[indicial_x_y_out_tmp[0][i]][indicial_x_y_out_tmp[1][i]]=background;
	}
}

void scan_conv::change_image_value (float value)
{
	int i=0;
	for (i=0 ; i<N_point_to_change ; i++)
	{
		Image[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]]=value;
	}
}

void scan_conv::change_image_value (image<float> &input, float value)
{
	int i=0;
	for (i=0 ; i<N_point_to_change ; i++)
	{
		input.data[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]]=value;
	}
}

void scan_conv::change_image_value (const image<float> &input, image<float> &output, float value)
{
    output = input;
	int i=0;
	for (i=0 ; i<N_point_to_change ; i++)
	{
		output.data[indicial_x_y_tmp[0][i]][indicial_x_y_tmp[1][i]]=value;
	}
}

void scan_conv::save_image(std::string name)
{
    std::ofstream txtfile;
    txtfile.open(name);
    if (!txtfile)
    {
        std::cout << "Error opening file " << name << " to write line<T>" << std::endl;
        exit(1);
    }

    //txtfile << Nel << std::endl;
    for (int i=0 ; i<Nx ; i++) 
    {
        for (int j=0 ; j<Ny ; j++)
        {
            txtfile << Image[i][j] << " ";
        }
        txtfile << std::endl;
    }
    txtfile.close();
}

void scan_conv::save_rmatrix(std::string name)
{
    std::ofstream txtfile;
    txtfile.open(name);
    if (!txtfile)
    {
        std::cout << "Error opening file " << name << " to write line<T>" << std::endl;
        exit(1);
    }

    //txtfile << Nel << std::endl;
    for (int i=0 ; i<Nx ; i++) 
    {
        for (int j=0 ; j<Ny ; j++)
        {
            txtfile << r_matrix[i][j] << " ";
        }
        txtfile << std::endl;
    }
    txtfile.close();
}

void scan_conv::save_tmatrix(std::string name)
{
    std::ofstream txtfile;
    txtfile.open(name);
    if (!txtfile)
    {
        std::cout << "Error opening file " << name << " to write line<T>" << std::endl;
        exit(1);
    }

    //txtfile << Nel << std::endl;
    for (int i=0 ; i<Nx ; i++) 
    {
        for (int j=0 ; j<Ny ; j++)
        {
            txtfile << theta_matrix[i][j] << " ";
        }
        txtfile << std::endl;
    }
    txtfile.close();
}
