#include<gtest/gtest.h>

#include"image.hpp"


/******
 * author: Jérôme Dubois
 * date: April 2020
 *
 *
 *
 *
 * ****/

double relative_error(double x, double y);

TEST( image_hpp_Constructor, size)
{
    //default size
    image<int> im1;
    EXPECT_EQ(1, im1.row_size());
    EXPECT_EQ(1, im1.column_size());
    //small size
    image<float> im2(12, 35);
    EXPECT_EQ(12, im2.row_size());
    EXPECT_EQ(35, im2.column_size());
    //big size
    image<double> im3(1234,1987);
    EXPECT_EQ(1234, im3.row_size());
    EXPECT_EQ(1987, im3.column_size());
}

TEST(image_hpp_Constructor, zero_initialisation)
{
    int nrow = 12, ncolumn = 87;
    //int
    image<int> im1(nrow, ncolumn);
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_EQ(0, im1.data[i][j]) << "image[" << i <<"][" << j <<"] not equal to 0";
        }
    }
    //float
    image<float> im2(nrow, ncolumn);
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_EQ(0.f, im2.data[i][j]) << "image[" << i <<"][" << j <<"] not equal to 0.f";
        }
    }
    //double
    image<double> im3(nrow, ncolumn);
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_EQ(0.0, im3.data[i][j]) << "line[" << i <<"][" << j <<"] not equal to 0.d";
        }
    }
}

TEST(image_hpp_Resize, size)
{
    //incresing both size
    image<int> im1;
    im1.resize(29,51);
    EXPECT_EQ(29, im1.row_size());
    EXPECT_EQ(51, im1.column_size());
    //decresing both size
    image<float> im2(512, 256);
    im2.resize(42, 13);
    EXPECT_EQ(42, im2.row_size());
    EXPECT_EQ(13, im2.column_size());
    //increasing one size while decreasing the other
    image<double> im3 (12, 256);
    im3.resize(128, 26);
    EXPECT_EQ(128, im3.row_size());
    EXPECT_EQ(26, im3.column_size());
    image<float> im4(256, 12);
    im4.resize(26, 128);
    EXPECT_EQ(26, im4.row_size());
    EXPECT_EQ(128, im4.column_size());
    //keeping the same size
    image<double> im5(51, 51);
    im5.resize(51, 51);
    EXPECT_EQ(51, im5.row_size());
    EXPECT_EQ(51, im5.column_size());

    //access to each data for valgrind test
    for (int i=0 ; i<im1.row_size() ; i++)
    {
        for (int j=0 ; j<im1.column_size() ; j++) {im1.data[i][j] = 12;}
    }
    for (int i=0 ; i<im2.row_size() ; i++)
    {
        for (int j=0 ; j<im2.column_size() ; j++) {im2.data[i][j] = 12;}
    }
    for (int i=0 ; i<im3.row_size() ; i++)
    {
        for (int j=0 ; j<im3.column_size() ; j++) {im3.data[i][j] = 12;}
    }
    for (int i=0 ; i<im4.row_size() ; i++)
    {
        for (int j=0 ; j<im4.column_size() ; j++) {im4.data[i][j] = 12;}
    }
    for (int i=0 ; i<im5.row_size() ; i++)
    {
        for (int j=0 ; j<im5.column_size() ; j++) {im5.data[i][j] = 12;}
    }
}

TEST(image_hpp_Resize, initialisation)
{
    //incresing size
    image<int> im1;
    im1.resize(23, 32);
    for (int i=0 ; i<23 ; i++)
    {
        for (int j=0 ; j<32; j++)
        {
            EXPECT_EQ(0, im1.data[i][j]) << "image[" << i << "][" << j << "] not equal to 0";
        }
    }
    //decresing size
    image<float> im2(512,256);
    im2.resize(42, 666);
    for (int i=0 ; i<42 ; i++)
    {
        for (int j=0 ; j<666 ; j++)
        {
            EXPECT_EQ(0.f, im2.data[i][j]) << "line[" << i << "][" << j << "] not equal to 0.f";
        }
    }
    //keeping the same size
    image<double> im3(51,69);
    im3.resize(51,69);
    for (int i=0 ; i<51 ; i++)
    {
        for (int j=0 ; j<69 ; j++)
        {
            EXPECT_EQ(0.0, im3.data[i][j]) << "image[" << i << "][" << j << "] not equal to 0.d";
        }
    }
}

TEST(image_hpp_Reset, T)
{
    int nrow = 69, ncolumn = 33;
    //int
    image<int> im1(nrow, ncolumn);
    for (int i=0 ; i<nrow ; i++) 
    {
        for (int j=0 ; j<ncolumn ; j++) {im1.data[i][j] = 2*i;}
    }
    im1.reset();
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_EQ(0, im1.data[i][j]) << "image[" << i <<"][" << j << "] not equal to 0";
        }
    }
    //float
    image<float> im2(nrow, ncolumn);
    for (int i=0 ; i<nrow ; i++) 
    { 
        for (int j=0 ; j<ncolumn ; j++)
        {
            im2.data[i][j] = 2.57f*float(i);
        }
    }
    im2.reset(5.684f);
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_EQ(5.684f, im2.data[i][j]) << "image[" << i <<"][" << j << "] not equal to reset target";
        }
    }
    //double
    image<double> im3(nrow, ncolumn);
    for (int i=0 ; i<nrow ; i++) 
    { 
        for (int j=0 ; j<ncolumn ; j++) {im3.data[i][j] = 23.12*double(i);}
    }
    im3.reset();
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_EQ(0.0, im3.data[i][j]) << "image[" << i <<"][" << j << "] not equal to 0.d";
        }
    }
}

TEST(image_hpp_Function, sum)
{
    int nrow = 5, ncolumn = 10;
    //int
    image<int> im1(nrow, ncolumn);
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            im1.data[i][j] = i;
        }
    }
    EXPECT_EQ(100, im1.sum()) << "int sum()";

    //float
    nrow = 56, ncolumn = 27;
    float value = 362.1f;
    image<float> im2(nrow, ncolumn);
    float fsum = float(nrow*ncolumn)*value;
    im2.reset(value);
    std::cout << "sum(): " << std::setprecision(15) << im2.sum() << std::endl;
    std::cout << "sumd(): " << std::setprecision(15) << im2.sumd() << std::endl;
    std::cout << "multiplication: " << std::setprecision(15) << fsum << std::endl;

    EXPECT_FLOAT_EQ(fsum, im2.sumd()) << "float sum()";
    //this test is made on sumd (double precision sum) because float sum is quite imprecise...

    //double
    nrow = 12345, ncolumn = 987;
    image<double> im3(nrow, ncolumn);
    int loop=0;
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            im3.data[i][j] = 1.68 * double(loop++); // arithmetic progression
        }
    }
    double dsum = float(nrow*ncolumn) * (im3.data[0][0] + im3.data[nrow-1][ncolumn-1])/2.0;
    EXPECT_DOUBLE_EQ(dsum, im3.sum()) << "double sum()";
}

TEST(image_hpp_Operator, left_affectation)
{
    float value = 0.f;
    int npoint = 72;
    image<float> im(npoint, npoint);
    //affecting correctly all line elements
    for (int i=0 ; i<npoint ; i++)
    {
        for (int j=0 ; j<npoint ; j++)
        {
            value = float (i*j) * 12.3f - 2.59f;
            im(i,j) = value;
            EXPECT_EQ(value, im.data[i][j]) << "image[" << i <<"][" << j << "] not equal to value";
        }
    }
    //overflow, negative ID, must affect to 0
    im.reset();
    value = 1268.542f;
    im(-9,-9) = value;
    EXPECT_EQ(value, im.data[0][0]);
    im.reset();
    im(-9,12) = value;
    EXPECT_EQ(value, im.data[0][12]);
    im.reset();
    im(15,-9) = value;
    EXPECT_EQ(value, im.data[15][0]);
    //overflow, ID >= npoint, must affect to ID npoint-1
    im.reset();
    value = 0.123456f;
    im(15*npoint,15*npoint) = value;
    EXPECT_EQ(value, im.data[npoint-1][npoint-1]);
    im.reset();
    im(15*npoint, npoint/2) = value;
    EXPECT_EQ(value, im.data[npoint-1][npoint/2]);
    im.reset();
    im(npoint/2, 2*npoint) = value;
    EXPECT_EQ(value, im.data[npoint/2][npoint-1]);
}

TEST(line_hpp_Operator, right_affectation)
{
    float value = 0.f;
    int npoint = 72;
    image<float> im(npoint, npoint);
    //affecting correctly all line elements
    for (int i=0 ; i<npoint ; i++)
    {
        for (int j=0 ; j<npoint ; j++)
        {
            im.data[i][j] = float (i*j) * 12.3f - 2.59f;
            value = im.data[i][j];
            EXPECT_EQ(value, im.data[i][j]) << "image[" << i <<"][" << j << "] not equal to value";
        }
    }
    //overflow, negative ID, must affect to 0
    value = im(-9,-9);
    EXPECT_EQ(value, im.data[0][0]);
    value = im(-9, npoint/3);
    EXPECT_EQ(value, im.data[0][npoint/3]);
    value = im(npoint/2, -9);
    EXPECT_EQ(value, im.data[npoint/2][0]);
    //overflow, ID >= npoint, must affect to ID npoint-1
    value = im(15*npoint,5*npoint);
    EXPECT_EQ(value, im.data[npoint-1][npoint-1]);
    value = im(npoint, npoint/4);
    EXPECT_EQ(value, im.data[npoint-1][npoint/4]);
    value = im(npoint/6, npoint+12);
    EXPECT_EQ(value, im.data[npoint/6][npoint-1]);
}

/*
double relative_error(double x, double y)
{
    double output = 0.0;
    if ( x == 0.0 ) {return y;}
    else if (y == 0.0 ) {return x;}
    else if ( std::abs(x)<std::abs(y) )
    {
        output = std::abs((y - x)/x); //maximum relative error > 0 
    } 
    else
    {
        output = std::abs((x - y)/y); //maximum relative error > 0
    }
    return output;
}

int main(int argc, char **argv)
{
    ::testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}
*/
