#include<gtest/gtest.h>

#include"line.hpp"


/******
 * author: Jérôme Dubois
 * date: March 2020
 *
 *
 *
 *
 * ****/

double relative_error(double x, double y);

TEST( line_hpp_Constructor, size)
{
    //default size
    line<int> line1;
    EXPECT_EQ(1, line1.size());
    //small size
    line<float> line2(12);
    EXPECT_EQ(12, line2.size());
    //big size
    line<double> line3(123456);
    EXPECT_EQ(123456, line3.size());
}

TEST(line_hpp_Constructor, zero_initialisation)
{
    int npoint = 12;
    //int
    line<int> line1(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(0, line1.data[i]) << "line[" << i <<"] not equal to 0";
    }
    //float
    line<float> line2(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(0.f, line2.data[i]) << "line[" << i <<"] not equal to 0.f";
    }
    //double
    line<double> line3(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(0.d, line3.data[i]) << "line[" << i <<"] not equal to 0.d";
    }
}

TEST(line_hpp_Resize, size)
{
    //incresing size
    line<int> line1;
    line1.resize(23);
    EXPECT_EQ(23, line1.size());
    //decresing size
    line<float> line2(512);
    line2.resize(42);
    EXPECT_EQ(42, line2.size());
    //keeping the same size
    line<double> line3(51);
    line3.resize(51);
    EXPECT_EQ(51, line3.size());
}

TEST(line_hpp_Resize, initialisation)
{
    //incresing size
    line<int> line1;
    line1.resize(23);
    for (int i=0 ; i<23 ; i++)
    {
        EXPECT_EQ(0, line1.data[i]) << "line[" << i <<"] not equal to 0";
    }
    //decresing size
    line<float> line2(512);
    line2.resize(42);
    for (int i=0 ; i<42 ; i++)
    {
        EXPECT_EQ(0.f, line2.data[i]) << "line[" << i <<"] not equal to 0.f";
    }
    //keeping the same size
    line<double> line3(51);
    line3.resize(51);
    for (int i=0 ; i<51 ; i++)
    {
        EXPECT_EQ(0.d, line3.data[i]) << "line[" << i <<"] not equal to 0.d";
    }
}

TEST(line_hpp_Reset, T)
{
    int npoint = 69;
    //int
    line<int> line1(npoint);
    for (int i=0 ; i<npoint ; i++) { line1.data[i] = 2*i;}
    line1.reset();
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(0, line1.data[i]) << "line[" << i <<"] not equal to 0";
    }
    //float
    line<float> line2(npoint);
    for (int i=0 ; i<npoint ; i++) { line2.data[i] = 2.57f*float(i);}
    line2.reset();
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(0.f, line2.data[i]) << "line[" << i <<"] not equal to 0.f";
    }
    //double
    line<double> line3(npoint);
    for (int i=0 ; i<npoint ; i++) { line3.data[i] = 23.12d*double(i);}
    line3.reset();
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(0.d, line3.data[i]) << "line[" << i <<"] not equal to 0.d";
    }
}

TEST(line_hpp_Function, sum)
{
    int npoint = 5;
    //int
    line<int> line1(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = i;
    }
    EXPECT_EQ(10, line1.sum()) << "int sum()";

    //float
    npoint = 56;
    line<float> line2(npoint);
    float fsum = float(npoint)*2.718f;
    for (int i=0 ; i<npoint ; i++)
    {
        line2.data[i] = 2.718f;
    }
    EXPECT_FLOAT_EQ(fsum, line2.sum()) << "float sum()";

    //double
    npoint = 12345;
    line<double> line3(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line3.data[i] = 5.68d * double(i); // arithmetic progression
    }
    double dsum = float(npoint) * (line3.data[0] + line3.data[npoint-1])/2.d;
    EXPECT_DOUBLE_EQ(dsum, line3.sum()) << "double sum()";
}

TEST(line_hpp_Function, min)
{
    //int
    int npoint = 73;
    line<int> line1(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = 35 - i;
    }
    EXPECT_EQ(-37, line1.min());
    line1.data[35] = -84;
    EXPECT_EQ(-84, line1.min());

    //float
    line<float> line2(npoint);
    for (int i=0 ; i<npoint/2 ; i++)
    {
        line2.data[i] = -4.56f * float(i);
    }
    float fmin = -4.56f * float (npoint/2 -1);
    for (int i=npoint/2 ; i<npoint ; i++)
    {
        line2.data[i] = float(i);
    }
    EXPECT_EQ(fmin, line2.min());

    //double
    line<double> line3(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line3.data[i] = double(i);
    }
    EXPECT_EQ(0.d, line3.min());
}

TEST(line_hpp_Function, max)
{
    //int
    int npoint = 73;
    line<int> line1(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = 35 - i;
    }
    EXPECT_EQ(35, line1.max());
    line1.data[35] = 84;
    EXPECT_EQ(84, line1.max());

    //float
    line<float> line2(npoint);
    for (int i=0 ; i<npoint/2 ; i++)
    {
        line2.data[i] = -4.56f * float(i);
    }
    for (int i=npoint/2 ; i<npoint ; i++)
    {
        line2.data[i] = float(i);
    }
    EXPECT_EQ(float(npoint-1), line2.max());

    //double
    line<double> line3(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line3.data[i] = -double(i);
    }
    EXPECT_EQ(0.d, line3.max());
}

TEST(line_hpp_Function, mean)
{
    //if sum() passed the test, mean() should pass it
    //sum of arithmetic serie: N*(u_0+u_{n-1})/2
    //so mean is (u_0+u_{n-1})/2
    double dmean = 0.d;
    //int
    int npoint = 4560;
    line<int> line1(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = i;
    }
    dmean = (double(npoint-1) + 0.d)/2.d;
    EXPECT_EQ (int(dmean), line1.mean());
    //float
    line<float> line2(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line2.data[i] = -37.84f + 2.1f*float(i);
    }
    dmean = (double(-37.84f*2.f + 2.1f*double(npoint-1)))/2.d;
    EXPECT_FLOAT_EQ(float(dmean), line2.mean()); //the test not pass with npoint 456, but but pass for 4560 due to round
    //double
    line<double> line3(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line3.data[i] = 42.d - 56.12d * double(i);
    }
    dmean = (2.d*42.d - 56.12d * double(npoint-1))/2.d;
    EXPECT_DOUBLE_EQ(dmean, line3.mean());
}

TEST(line_hpp_Operator, left_affectation)
{
    float value = 0.f;
    int npoint = 72;
    line<float> line(npoint);
    //affecting correctly all line elements
    for (int i=0 ; i<npoint ; i++)
    {
        value = float (i) * 12.3f - 2.59f;
        line(i) = value;
        EXPECT_EQ(value, line.data[i]) << "line[" << i <<"] not equal to value";
    }
    //overflow, negative ID, must affect to 0
    value = 1268.542f;
    line(-9) = value;
    EXPECT_EQ(value, line.data[0]);
    //overflow, ID >= npoint, must affect to ID npoint-1
    value = 0.123456f;
    line(15*npoint) = value;
    EXPECT_EQ(value, line.data[npoint-1]);
}

TEST(line_hpp_Operator, right_affectation)
{
    float value = 0.f;
    int npoint = 72;
    line<float> line(npoint);
    //affecting correctly all line elements
    for (int i=0 ; i<npoint ; i++)
    {
        line.data[i] = float (i) * 12.3f - 2.59f;
        value = line.data[i];
        EXPECT_EQ(value, line.data[i]) << "line[" << i <<"] not equal to value";
    }
    //overflow, negative ID, must affect to 0
    value = line(-9);
    EXPECT_EQ(value, line.data[0]);
    //overflow, ID >= npoint, must affect to ID npoint-1
    value = line(15*npoint);
    EXPECT_EQ(value, line.data[npoint-1]);
}

TEST(line_hpp_Operator, scalar_plus)
{
    int npoint = 42;
    //int
    line<int> line1(npoint); // line1 is initialized at 0
    line1 + 4;
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(4, line1.data[i]) << "line[" << i <<"] not equal to ivalue";
    }
    //float
    line<float> line2(npoint);
    line<float> line3(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line2.data[i] = 2.36f*float(i);
        line3.data[i] = 2.36f*float(i+20);
    }
    line2 + 47.2f;
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_FLOAT_EQ(line3.data[i], line2.data[i]) << "line[" << i <<"] not equal to fvalue";
    }
    //double
    double dvalue = 0.d;
    line<double> line4(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line4.data[i] = -0.854d*double(i);
    }
    dvalue = double(npoint) * (line4.data[0] + line4.data[npoint-1]) / 2.d;
    line4 + 1.54d;
    dvalue += double(npoint) * 1.54d;
    EXPECT_DOUBLE_EQ(dvalue, line4.sum()); //is sum() is validated this must pass
}

TEST(line_hpp_Operator, scalar_minus)
{
    int npoint = 68;
    //int
    line<int> line1(npoint);
    line1 - 12;
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(-12, line1.data[i]) << "line[" << i <<"] not equal to ivalue";
    }
    //float
    float fvalue = 154.256f;
    line<float> line2(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line2.data[i] = fvalue;
    }
    line<float> line3 = line2;
    line2 - 4.3845682f;
    line3 + 4.3845682f;
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_FLOAT_EQ(2.f*fvalue, line2.data[i]+line3.data[i]) << "line[" << i <<"] not equal to fvalue"; //if scalar + is validated, this test must pass
    }
    //double
    double dvalue = 0.d;
    line<double> line4(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line4.data[i] = -0.951248d*double(i);
    }
    dvalue = double(npoint) * (line4.data[0] + line4.data[npoint-1]) / 2.d;
    line4 - 1.54d; 
    dvalue -= double(npoint) * 1.54d;
    EXPECT_DOUBLE_EQ(dvalue, line4.sum()); //if sum() has passed, this must pass
}

TEST(line_hpp_Operator, scalar_multiplication)
{
    int npoint = 42;
    //int
    line<int> line1(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = i;
    }
    line1 * 0;
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(0 , line1.data[i]) << "line[" << i <<"] not equal to ivalue";
    }
    //float
    float fvalue = 0.f;
    line<float> line2(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line2.data[i] = float(i)/2.15f;
    }
    line2 * 4.3f; //trying the automotique cast into int
    for (int i=0 ; i<npoint ; i++)
    {
        fvalue = float(i) * 2.f;
        EXPECT_FLOAT_EQ(fvalue, line2.data[i]) << "line[" << i <<"] not equal to fvalue";
    }
    //double
    npoint = 84261;
    double dvalue = 0.d;
    line<double> line3(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line3.data[i] = 1.2569d*double(i);
    }
    dvalue = double(npoint) * (line3.data[0] + line3.data[npoint-1]) / 2.d;
    line3 * 1.5469d; //trying the automotique cast into int
    dvalue *= 1.5469d;
    EXPECT_DOUBLE_EQ(dvalue, line3.sum()); // with less point it say's nok, but error is < 10^(-13)
}

TEST(line_hpp_Operator, scalar_division)
{
    int npoint = 42;
    //int
    line<int> line1(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = i;
    }
    line1 / 1;
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(i, line1.data[i]) << "line[" << i <<"] not equal to ivalue";
    }
    //float
    float fvalue = 0.f;
    line<float> line2(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line2.data[i] = 4.3f*float(i);
    }
    line2 / 2.15f;
    for (int i=0 ; i<npoint ; i++)
    {
        fvalue = 2.f*float(i);
        EXPECT_FLOAT_EQ(fvalue, line2.data[i]) << "line[" << i <<"] not equal to fvalue";
    }
    //double
    npoint = 84261;
    double dvalue = 0.d;
    line<double> line3(npoint);
    for (int i=0 ; i<npoint ; i++)
    {
        line3.data[i] = -0.0059332d*double(i);
    }
    dvalue = double(npoint) * (line3.data[0] + line3.data[npoint-1]) / 2.d;
    line3 / 154.154d;
    dvalue /= 154.154d;
    EXPECT_DOUBLE_EQ(dvalue, line3.sum());
}

TEST(line_hpp_Operator, vectorial_plus)
{
    int npoint = 99;
    
    //different size, second greater make only with int
    line<int> line1(npoint), line2(npoint+12), line3;
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = i + 69;
        line2.data[i] = -5*i + 12;
    }
    line3 = line1 + line2;
    EXPECT_EQ(line3.size(), line1.size()) << "sum line not the same size than lhs";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(line3.data[i], line1.data[i]) << "int line[" << i << "] not equal to sum";
    }
    //different size, second smaller
    int npoint2 = npoint-15;
    line2.resize(npoint2);
    line3.resize(1);
    for (int i=0 ; i<npoint2 ; i++)
    {
        line1.data[i] = i + 69;
        line2.data[i] = -5*i + 12;
    }
    line3 = line1 + line2;
    EXPECT_EQ(line3.size(), line1.size()) << "sum line not the same size than lhs";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(line3.data[i], line1.data[i]) << "int line[" << i << "] not equal to sum";
    }

    //same size
    //int
    line<int> linei1(npoint), linei2(npoint), linei3(1);
    for (int i=0 ; i<npoint ; i++)
    {
        linei2.data[i] = i + 69;
    }
    linei3 = linei1 + linei2;
    EXPECT_EQ(linei2.size(), linei3.size()) << "sum line not the same size than over line<int>";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(linei2.data[i], linei3.data[i]) << "int line[" << i << "] not equal to sum";
    }
    //float
    line<float> linef1(npoint), linef2(npoint), linef3(5);
    for (int i=0 ; i<npoint ; i++)
    {
        linef1.data[i] = float(i) + 69.69f;
        linef2.data[i] = -float(i) + 12.42f;
    }
    linef3 = linef1 + linef2;
    EXPECT_EQ(linef3.size(), linef2.size()) << "sum line not the same size than over line<float>";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_FLOAT_EQ(linef3.data[i], 69.69f+12.42f) << "float line[" << i << "] not equal to sum";
    }
    //double
    npoint *= 10;
    double dvalue = 0.0;
    line<double> lined1(npoint), lined2(npoint), lined3(5);
    for (int i=0 ; i<npoint ; i++)
    {
        lined1.data[i] = 1.25*double(i) + 4.258;
        lined2.data[i] = 1.25*double(i+npoint) + 4.258;
    }
    lined3 = lined1 + lined2;
    dvalue = double(2*npoint) * (lined1.data[0] + lined2.data[npoint-1]) / 2.0;
    EXPECT_EQ(lined3.size(), lined2.size()) << "sum line not the same size than over line<float>";
    //EXPECT_DOUBLE_EQ(dvalue, lined3.sum());
    //EXPECT_NEAR(dvalue, lined3.sum(), 1.0e-8); work with absolute error, not relative
    EXPECT_LE(relative_error(dvalue, lined3.sum()), 1e-12);
}

TEST(line_hpp_Operator, vectorial_minus)
{
    int npoint = 99;
    
    //different size, second greater make only with int
    line<int> line1(npoint), line2(npoint+12), line3;
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = i + 69;
        line2.data[i] = -5*i + 12;
    }
    line3 = line1 - line2;
    EXPECT_EQ(line3.size(), line1.size()) << "minus line not the same size than lhs";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(line3.data[i], line1.data[i]) << "int line[" << i << "] not equal to minus";
    }
    //different size, second smaller
    int npoint2 = npoint-15;
    line2.resize(npoint2);
    line3.resize(1);
    for (int i=0 ; i<npoint2 ; i++)
    {
        line1.data[i] = i + 69;
        line2.data[i] = -5*i + 12;
    }
    line3 = line1 - line2;
    EXPECT_EQ(line3.size(), line1.size()) << "minus line not the same size than lhs";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(line3.data[i], line1.data[i]) << "int line[" << i << "] not equal to minus";
    }

    //same size
    //int
    line<int> linei1(npoint), linei2(npoint), linei3(1);
    for (int i=0 ; i<npoint ; i++)
    {
        linei1.data[i] = i + 69;
    }
    linei2 = linei1;
    linei3 = linei2 - linei1;
    EXPECT_EQ(linei3.size(), linei2.size()) << "minus line not the same size than over line<int>";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(linei3.data[i], 0) << "int line[" << i << "] not equal to minus";
    }
    //float
    line<float> linef1(npoint), linef2(npoint), linef3(5);
    for (int i=0 ; i<npoint ; i++)
    {
        linef1.data[i] = float(i) + 69.69f;
        linef2.data[i] = float(i);
    }
    linef3 = linef1 - linef2;
    EXPECT_EQ(linef3.size(), linef2.size()) << "minus line not the same size than over line<float>";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_FLOAT_EQ(69.69f, linef3.data[i]) << "float line[" << i << "] not equal to minus";
    }
    //double
    npoint *= 10;
    double dvalue = 0.0;
    line<double> lined1(npoint), lined2(npoint), lined3(5);
    for (int i=0 ; i<npoint ; i++)
    {
        lined1.data[i] = 1.25*double(i) + 4.258;
        lined2.data[i] = -1.25*double(i+npoint) - 4.258;
    }
    lined3 = lined1 - lined2;
    dvalue = double(2*npoint) * (lined1.data[0] - lined2.data[npoint-1]) / 2.0;
    EXPECT_EQ(lined3.size(), lined2.size()) << "sum line not the same size than over line<float>";
    EXPECT_LE(relative_error(dvalue, lined3.sum()), 1e-12);
}

TEST(line_hpp_Operator, vectorial_multiply)
{
    int npoint = 99;
    
    //different size, second greater make only with int
    line<int> line1(npoint), line2(npoint+12), line3;
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = i + 69;
        line2.data[i] = -5*i + 12;
    }
    line3 = line1 * line2;
    EXPECT_EQ(line3.size(), line1.size()) << "* line not the same size than lhs";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(line3.data[i], line1.data[i]) << "int line[" << i << "] not equal to *";
    }
    //different size, second smaller
    int npoint2 = npoint-15;
    line2.resize(npoint2);
    line3.resize(1);
    for (int i=0 ; i<npoint2 ; i++)
    {
        line1.data[i] = i + 69;
        line2.data[i] = -5*i + 12;
    }
    line3 = line1 * line2;
    EXPECT_EQ(line3.size(), line1.size()) << "* line not the same size than lhs";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(line3.data[i], line1.data[i]) << "int line[" << i << "] not equal to *";
    }

    //same size
    //int
    line<int> linei1(npoint), linei2(npoint), linei3(1);
    for (int i=0 ; i<npoint ; i++)
    {
        linei1.data[i] = 1.f;
        linei2.data[i] = i;
    }
    linei3 = linei1 * linei2;
    EXPECT_EQ(linei3.size(), linei2.size()) << "* line not the same size than over line<int>";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(i, linei3.data[i]) << "int line[" << i << "] not equal to *";
    }
    //float
    line<float> linef1(npoint), linef2(npoint), linef3(5);
    float fvalue = 0.f;
    for (int i=0 ; i<npoint ; i++)
    {
        linef1.data[i] = float(i) + 69.69f;
        linef2.data[i] = float(i) + 69.69f;
    }
    linef3 = linef1 * linef2;
    EXPECT_EQ(linef3.size(), linef2.size()) << "* line not the same size than over line<float>";
    for (int i=0 ; i<npoint ; i++)
    {
        fvalue = (float(i) + 69.69f) * (float(i) + 69.69f);
        EXPECT_FLOAT_EQ(fvalue, linef3.data[i]) << "float line[" << i << "] not equal to *";
    }
    //double
    double dvalue = 0.0;
    line<double> lined1(npoint), lined2(npoint), lined3(5);
    for (int i=0 ; i<npoint ; i++)
    {
        lined1.data[i] = 1.357;
        lined2.data[i] = -0.19436*double(i);
    }
    lined3 = lined1 * lined2;
    EXPECT_EQ(lined3.size(), lined2.size()) << "* line not the same size than over line<float>";
    dvalue = lined1.data[0] * double(npoint) * (lined2.data[0] + lined2.data[npoint-1]) / 2.0;
    EXPECT_LE(relative_error(dvalue, lined3.sum()), 1e-12);
}

TEST(line_hpp_Operator, vectorial_divide)
{
    int npoint = 99;
    
    //different size, second greater make only with int
    line<int> line1(npoint), line2(npoint+12), line3;
    for (int i=0 ; i<npoint ; i++)
    {
        line1.data[i] = i + 69;
        line2.data[i] = -5*i + 12;
    }
    line3 = line1 / line2;
    EXPECT_EQ(line3.size(), line1.size()) << "* line not the same size than lhs";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(line3.data[i], line1.data[i]) << "int line[" << i << "] not equal to *";
    }
    //different size, second smaller
    int npoint2 = npoint-15;
    line2.resize(npoint2);
    line3.resize(1);
    for (int i=0 ; i<npoint2 ; i++)
    {
        line1.data[i] = i + 69;
        line2.data[i] = -5*i + 12;
    }
    line3 = line1 / line2;
    EXPECT_EQ(line3.size(), line1.size()) << "* line not the same size than lhs";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(line3.data[i], line1.data[i]) << "int line[" << i << "] not equal to *";
    }

    //same size
    //int
    line<int> linei1(npoint), linei2(npoint), linei3(1);
    for (int i=0 ; i<npoint ; i++)
    {
        linei1.data[i] = i + 69;
    }
    linei3 = linei2 / linei1;
    EXPECT_EQ(linei3.size(), linei2.size()) << "* line not the same size than over line<int>";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_EQ(0, linei3.data[i]) << "int line[" << i << "] not equal to *";
    }
    //float
    line<float> linef1(npoint), linef2(5);
    for (int i=0 ; i<npoint ; i++)
    {
        linef1.data[i] = -5.1f*float(i) + 12.42f;
    }
    linef2 = linef1 / linef1;
    EXPECT_EQ(linef1.size(), linef2.size()) << "* line not the same size than over line<float>";
    for (int i=0 ; i<npoint ; i++)
    {
        EXPECT_FLOAT_EQ(1.f, linef2.data[i]) << "float line[" << i << "] not equal to *";
    }
    //double
    double dvalue = 0.0;
    line<double> lined1(npoint), lined2(npoint), lined3(5);
    for (int i=0 ; i<npoint ; i++)
    {
        lined1.data[i] = 1.357;
        lined2.data[i] = -0.19436*double(i+1);
    }
    lined3 = lined2 / lined1;
    EXPECT_EQ(lined3.size(), lined2.size()) << "/ line not the same size than over line<float>";
    dvalue = (1.0/lined1.data[0]) * double(npoint) * (lined2.data[0] + lined2.data[npoint-1]) / 2.0;
    std::cout << "dvalue: " << dvalue << ", sum: " << lined3.sum() << std::endl;
    EXPECT_LE(relative_error(dvalue, lined3.sum()), 1e-12);
}

double relative_error(double x, double y)
{
    double output = 0.0;
    if ( x == 0.0 ) {return y;}
    else if (y == 0.0 ) {return x;}
    else if ( std::abs(x)<std::abs(y) )
    {
        output = std::abs((y - x)/x); //maximum relative error > 0 
    } 
    else
    {
        output = std::abs((x - y)/y); //maximum relative error > 0
    }
    return output;
}

/*
int main(int argc, char **argv)
{
    ::testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}
*/
