#include<gtest/gtest.h>

#include<random> // need c++11
#include<cmath>
#include<iostream>

#include"image.hpp"
#include"SigProc2D.hpp"

/*********
author: Jérôme Dubois
date: April 2020



********/

TEST( Sigproc2D, GrayScale_settings)
{
    float lb = 0.f, ub = 0.f;
    int gl = 5;
    QuantificationType lt = sigmoid;
    SigProc2D sp2;

    sp2.edit_LUT_bound(lb, ub);
    EXPECT_EQ(0.f, sp2.get_LUT_lower_bound());
    EXPECT_EQ(1.f, sp2.get_LUT_upper_bound());
    lb = 0.26f, ub = 103.56f;
    sp2.edit_LUT_bound(lb, ub);
    EXPECT_EQ(lb, sp2.get_LUT_lower_bound());
    EXPECT_EQ(ub, sp2.get_LUT_upper_bound());

    sp2.edit_LUT_gray_level(-2);
    EXPECT_EQ(1, sp2.get_LUT_gray_level());
    sp2.edit_LUT_gray_level(15);
    EXPECT_EQ(8, sp2.get_LUT_gray_level());
    sp2.edit_LUT_gray_level(gl);
    EXPECT_EQ(gl, sp2.get_LUT_gray_level());

    sp2.edit_LUT_quantification_type(lt);
    EXPECT_EQ(lt, sp2.get_LUT_quantification_type());
}

TEST( SigProc2D, GrayScale_QuantificationType)
{
    int nr = 24, nc = 36;
    image<float> input(nr,nc);
    image<int> output;
    SigProc2D sp2;
    sp2.edit_LUT_thread_number(1);
    for (int i=0 ; i<nr ; i++)
    {
        for (int j=0 ; j<nc ; j++) {input(i,j) = float(i+j);}
    }
    sp2.edit_LUT_bound(0.f, 255.f); //with this bound, output = int(input)
    sp2.apply_LUT(input, output);
    for (int i=0 ; i<nr ; i++)
    {
        for (int j=0 ; j<nc ; j++)
        {
            EXPECT_EQ(int(input(i,j)), output(i,j));
        }
    }
    nc = 512, nr = 1;
    input.resize(nr,nc);
    for (int i=0 ; i<nc ; i++)
    {
        input(0,i) = float(i)-100.f;
    }
    sp2.apply_LUT(input, output);
    for (int i=0 ; i<100 ; i++)
    {
        EXPECT_EQ(0, output(0,i));
    }
    for (int i=100 ; i<355 ; i++)
    {
        EXPECT_EQ(i-100, output(0,i));
    }
    for (int i=355 ; i<nc; i++)
    {
        EXPECT_EQ(255, output(0,i));
    }

    //logaritm
    nc = 2048;
    input.resize(nr, nc);
    image<int> ref(nr,nc);
    for (int i=0 ; i<nc ; i++)
    {
        input(0,i) = float(i);
    }
    sp2.edit_LUT_bound(0.f, input(0,nc-1));
    sp2.edit_LUT_quantification_type(logaritmic);

    std::ifstream txtfile;
    txtfile.open("./octave_simulation/log_lut.txt");
    if (txtfile.is_open())
    {
        for (int i=0 ; i<nc ; i++)
        {
            txtfile >> input(0,i)  >> ref(0,i);
        }
        txtfile.close();
    }
    else
    {
        std::cerr << "Warning, can't open ./octave_simulation/log_lut.txt" << std::endl;
        EXPECT_EQ(0,1); // don't pass the test if we can't open the file
    }

    sp2.apply_LUT(input, output);

    for (int i=0 ; i<nc ; i++)
    {
        EXPECT_EQ(output(0,i), ref(0,i));
    }

    //exponential
    txtfile.open("./octave_simulation/exp_lut.txt");
    if (txtfile.is_open())
    {
        for (int i=0 ; i<nc ; i++)
        {
            txtfile >> input(0,i)  >> ref(0,i);
        }
        txtfile.close();
    }
    else
    {
        std::cout << "Warning, can't open ./octave_simulation/exp_lut.txt" << std::endl;
        EXPECT_EQ(0,1);
    }
    sp2.edit_LUT_quantification_type(exponential);
    sp2.edit_LUT_bound(0.f, input(0,nc-1));
    sp2.apply_LUT(input, output);
    for (int i=0 ; i<nc ; i++)
    {
        EXPECT_EQ(output(0,i), ref(0,i));
    }

    // sigmoid
    txtfile.open("./octave_simulation/sig_lut.txt");
    if (txtfile.is_open())
    {
        for (int i=0 ; i<nc ; i++)
        {
            txtfile >> input(0,i)  >> ref(0,i);
        }
        txtfile.close();
    }
    else
    {
        std::cerr << "Warning, can't open ./octave_simulation/sig_lut.txt" << std::endl;
        EXPECT_EQ(0,1);
    }
    sp2.edit_LUT_quantification_type(sigmoid);
    sp2.edit_LUT_bound(0.f, input(0,nc-1));
    sp2.apply_LUT(input, output);
    for (int i=0 ; i<nc ; i++)
    {
        EXPECT_EQ(output(0,i), ref(0,i));
    }
}

TEST( Sigproc2D, GrayScale_GrayScale)
{
    int nr = 1, nc = 256;
    image<float> input(nr,nc);
    image<int> output, ref(nr, nc);
    SigProc2D sp2;
    sp2.edit_LUT_thread_number(8);

    // linear 2 level output
    std::ifstream txtfile;
    txtfile.open("./octave_simulation/lin_lut_2l.txt");
    if (txtfile.is_open())
    {
        for (int i=0 ; i<nc ; i++)
        {
            txtfile >> input(0,i)  >> ref(0,i);
        }
        txtfile.close();
    }
    else
    {
        std::cerr << "Warning, can't open ./octave_simulation/lin_lut_2l.txt" << std::endl;
        EXPECT_EQ(0,1); 
    }
    sp2.edit_LUT_quantification_type(linear);
    sp2.edit_LUT_bound(input(0,0), input(0,nc-1));
    sp2.edit_LUT_gray_level(1);
    sp2.apply_LUT(input, output);
    for (int i=0 ; i<nc ; i++)
    {
        EXPECT_EQ(output(0,i), ref(0,i)) << "linear lut 2 level";
    }

    // exponential 4 level output
    txtfile.open("./octave_simulation/exp_lut_4l.txt");
    if (txtfile.is_open())
    {
        for (int i=0 ; i<nc ; i++)
        {
            txtfile >> input(0,i)  >> ref(0,i);
        }
        txtfile.close();
    }
    else
    {
        std::cerr << "Warning, can't open ./octave_simulation/exp_lut_4l.txt" << std::endl;
        EXPECT_EQ(0,1);
    }
    sp2.edit_LUT_quantification_type(exponential);
    sp2.edit_LUT_bound(input(0,0), input(0,nc-1));
    sp2.edit_LUT_gray_level(2);
    sp2.apply_LUT(input, output);
    for (int i=0 ; i<nc ; i++)
    {
        EXPECT_EQ(output(0,i), ref(0,i)) << "exp lut 4 level";
    }

    // logaritmic 8 level output
    txtfile.open("./octave_simulation/log_lut_8l.txt");
    if (txtfile.is_open())
    {
        for (int i=0 ; i<nc ; i++)
        {
            txtfile >> input(0,i)  >> ref(0,i);
        }
        txtfile.close();
    }
    else
    {
        std::cerr << "Warning, can't open ./octave_simulation/log_lut_8l.txt" << std::endl;
        EXPECT_EQ(0,1);
    }
    sp2.edit_LUT_quantification_type(logaritmic);
    sp2.edit_LUT_bound(input(0,0), input(0,nc-1));
    sp2.edit_LUT_gray_level(3);
    sp2.apply_LUT(input, output);
    for (int i=0 ; i<nc ; i++)
    {
        EXPECT_EQ(output(0,i), ref(0,i)) << "log lut 8 level";
    }

    // sigmoid 16 level output
    txtfile.open("./octave_simulation/sig_lut_16l.txt");
    if (txtfile.is_open())
    {
        for (int i=0 ; i<nc ; i++)
        {
            txtfile >> input(0,i)  >> ref(0,i);
        }
        txtfile.close();
    }
    else
    {
        std::cerr << "Warning, can't open ./octave_simulation/sig_lut_16l.txt" << std::endl;
        EXPECT_EQ(0,1);
    }
    sp2.edit_LUT_quantification_type(sigmoid);
    sp2.edit_LUT_bound(input(0,0), input(0,nc-1));
    sp2.edit_LUT_gray_level(4);
    sp2.apply_LUT(input, output);
    for (int i=0 ; i<nc ; i++)
    {
        EXPECT_EQ(output(0,i), ref(0,i)) << "sig lut 16 level";
    }
}

TEST( Sigproc2D, Filter_settings)
{
    float f00=0.f, f01=1.f, f02=2.f, f10=10.f, f11=11.f, f12=12.f, f20=20.f, f21=21.f, f22=22.f;
    image<float> lkernel(3,3), gkernel;
    for (int i=0 ; i<3 ; i++)
    {
        for (int j=0 ; j<3 ; j++)
        {
            lkernel(i,j) = 1.f;
        }
    }
    FilterType ft = convolution;
    SigProc2D sp2;
    gkernel = sp2.get_filter_kernel();

    EXPECT_EQ(no_filter, sp2.get_filter_type());
    for (int i=0 ; i<3 ; i++)
    {
        for (int j=0 ; j<3 ; j++)
        {
            EXPECT_EQ(lkernel(i,i)/9.f,gkernel(i,j)); 
        }
    }

    for (int i=0 ; i<3 ; i++)
    {
        for (int j=0 ; j<3 ; j++)
        {
            lkernel(i,j) = float(i+j);
        }
    }
    sp2.edit_filter_kernel(lkernel);
    gkernel=sp2.get_filter_kernel();
    for (int i=0 ; i<3 ; i++)
    {
        for (int j=0 ; j<3 ; j++)
        {
            EXPECT_EQ(lkernel(i,j)/lkernel.sum(), gkernel(i,j));
        }
    }
    lkernel(0,0)=f00;
    lkernel(0,1)=f01;
    lkernel(0,2)=f02;
    lkernel(1,0)=f10;
    lkernel(1,1)=f11;
    lkernel(1,2)=f12;
    lkernel(2,0)=f20;
    lkernel(2,1)=f21;
    lkernel(2,2)=f22;
    sp2.edit_filter_kernel(f00, f01, f02, f10, f11, f12, f20, f21, f22);
    gkernel=sp2.get_filter_kernel();

    for (int i=0 ; i<3 ; i++)
    {
        for (int j=0 ; j<3 ; j++)
        {
            EXPECT_EQ(lkernel(i,j)/lkernel.sum(), gkernel(i,j));
        }
    }

    sp2.edit_filter_type(ft);
    EXPECT_EQ(ft, sp2.get_filter_type());
}

TEST( Sigproc2D, Filter)
{
    int nrow = 128/8, ncolumn = 256/8;
    image<float> input(nrow, ncolumn), output, expected(nrow, ncolumn), lkernel(3,3);
    SigProc2D sp2;
    sp2.edit_filter_thread_number(8);

    // prepare random generetor c++11
    std::random_device rd;
    std::mt19937 e2(rd());
    std::uniform_real_distribution<> dist(0.1f, 10.f);
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            input(i,j) = dist(e2);
        }
    }

    // default: no filter
    sp2.apply_filter(input, output);
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_EQ(input(i,j), output(i,j));
        }
    }

    lkernel(1,1)=23.482f;
    sp2.edit_filter_type(convolution);
    sp2.edit_filter_kernel(lkernel);
    sp2.apply_filter(input, output);
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_EQ(input(i,j), output(i,j));
        }
    }

    nrow = 12*3, ncolumn = 56*3;
    input.resize(12*3, 56*3);
    input.reset();
    for (int i=1 ; i<nrow ; i+=3)
    {
        for (int j=1 ; j<ncolumn ; j+=3)
        {   
            input(i,j)=1.f;
        }
    }
    sp2.edit_filter_type(median);
    sp2.apply_filter(input, output);
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_EQ(0.f, output(i,j));
        }
    }

    nrow = 128/8, ncolumn = 256/8;
    input.resize(nrow, ncolumn);
    int val = 1;
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            val = !val;
            input(i,j) = float(val);
        }
        val = !val;
    }

    for (int i=0 ; i<3 ; i++)
    {
        for (int j=0 ; j<3 ; j++)
        {
            lkernel(i,j) = 2.14569f;
        }
    }
    sp2.edit_filter_type(convolution);
    sp2.edit_filter_kernel(lkernel);
    sp2.apply_filter(input, output);
    expected.resize(nrow, ncolumn);
    for (int i=0 ; i<nrow ; i++)
    {
        expected(i,0) = 0.5f;
        expected(i, ncolumn-1) = 0.5f;
    }
    for (int j=1 ; j<ncolumn-1 ; j++)
    {
        expected(0,j) = 0.5f;
        expected(nrow-1,j) = 0.5f;
    }
    for (int i=1 ; i<nrow-1 ; i++)
    {
        for (int j=1 ; j<ncolumn-1 ; j++)
        {
            expected(i,j) = (4.f+input(i,j))/9.f;
        }
    }
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_FLOAT_EQ(expected(i,j), output(i,j));
        }
    }

    sp2.edit_filter_type(median);
    sp2.apply_filter(input, output);
    expected.reset();
    for (int i=1 ; i<nrow-1 ; i++)
    {
        for (int j=1 ; j<ncolumn-1 ; j++)
        {
            expected(i,j) = input(i,j);
        }
    }
    for (int i=0 ; i<nrow ; i++)
    {
        for (int j=0 ; j<ncolumn ; j++)
        {
            EXPECT_FLOAT_EQ(expected(i,j), output(i,j));
        }
    }
}

TEST(toto, tutu)
{
    SigProc2D sp2;
    //sp2.set_scan_conv_image_square();
    //sp2.set_scan_conv_image_arbitrary();
    image<float> in(64,32);
    std::cout << "Nx = " << sp2.get_scan_conv_Nx() << ", Ny = " << sp2.get_scan_conv_Ny() << ", Nline = " << sp2.get_scan_conv_Nline() << ", Npoint = " << sp2.get_scan_conv_Npoint() << std::endl;
    sp2.edit_scan_conv_us_image_parameters(10,10);
    sp2.edit_scan_conv_image_size(64,32);
    sp2.calcul_scan_conv_kernel();
    sp2.scan_conv_change_image_background(in, 0.f);
    sp2.scan_conv_change_image_value(in, 1.f);
    std::cout << "image: " << in;
}

int main(int argc, char **argv)
{
    ::testing::InitGoogleTest(&argc, argv);
    return RUN_ALL_TESTS();
}
